import re
from enum import Enum
from datetime import datetime
from typing import Dict, List, Any, Union

from dateutil.parser import parse, ParserError
from dateutil import tz


class TLDBaseKeys(str, Enum):
    DOMAIN_NAME = "domain_name"

    CREATED = "created"
    UPDATED = "updated"
    EXPIRES = "expires"

    REGISTRAR = "registrar"
    REGISTRAR_IANA_ID = "registrar_iana_id"
    REGISTRAR_URL = "registrar_url"
    REGISTRAR_ABUSE_EMAIL = "registrar_abuse_email"
    REGISTRAR_ABUSE_PHONE = "registrar_abuse_phone"

    REGISTRANT_NAME = "registrant_name"
    REGISTRANT_ORGANIZATION = "registrant_organization"
    REGISTRANT_ADDRESS = "registrant_address"
    REGISTRANT_CITY = "registrant_city"
    REGISTRANT_STATE = "registrant_state"
    REGISTRANT_COUNTRY = "registrant_country"
    REGISTRANT_ZIPCODE = "registrant_zipcode"
    REGISTRANT_PHONE = "registrant_phone"
    REGISTRANT_FAX = "registrant_fax"
    REGISTRANT_EMAIL = "registrant_email"

    ADMIN_NAME = "admin_name"
    ADMIN_ID = "admin_id"
    ADMIN_ORGANIZATION = "admin_organization"
    ADMIN_ADDRESS = "admin_address"
    ADMIN_CITY = "admin_city"
    ADMIN_STATE = "admin_state"
    ADMIN_COUNTRY = "admin_country"
    ADMIN_ZIPCODE = "admin_zipcode"
    ADMIN_PHONE = "admin_phone"
    ADMIN_FAX = "admin_fax"
    ADMIN_EMAIL = "admin_email"

    BILLING_NAME = "billing_name"
    BILLING_ID = "billing_id"
    BILLING_ORGANIZATION = "billing_organization"
    BILLING_ADDRESS = "billing_address"
    BILLING_CITY = "billing_city"
    BILLING_STATE = "billing_state"
    BILLING_COUNTRY = "billing_country"
    BILLING_ZIPCODE = "billing_zipcode"
    BILLING_PHONE = "billing_phone"
    BILLING_FAX = "billing_fax"
    BILLING_EMAIL = "billing_email"

    TECH_NAME = "tech_name"
    TECH_ID = "tech_id"
    TECH_ORGANIZATION = "tech_organization"
    TECH_ADDRESS = "tech_address"
    TECH_CITY = "tech_city"
    TECH_STATE = "tech_state"
    TECH_COUNTRY = "tech_country"
    TECH_ZIPCODE = "tech_zipcode"
    TECH_PHONE = "tech_phone"
    TECH_FAX = "tech_fax"
    TECH_EMAIL = "tech_email"

    DNSSEC = "dnssec"
    STATUS = "status"
    NAME_SERVERS = "name_servers"

    def __repr__(self):
        return self.value

    def __str__(self):
        return self.value


class IPBaseKeys(str, Enum):
    NET_RANGE = "net_range"
    CIDR = "cidr"
    NET_NAME = "net_name"
    NET_TYPE = "net_type"
    NET_HANDLE = "net_handle"
    PARENT = "parent"
    ORIGIN_AS = "origin_as"
    ORGANIZATION = "organization"
    REG_DATE = "registered_date"
    UPDATED = "updated_date"
    RDAP_IP_REF = "rdap_ref"

    ORG_NAME = "org_name"
    ORG_ID = "org_id"
    ORG_ADDRESS = "org_address"
    ORG_CITY = "org_city"
    ORG_STATE = "org_state"
    ORG_COUNTRY = "org_country"
    ORG_ZIPCODE = "org_zipcode"
    ORG_REG_DATE = "org_registered_date"
    ORG_UPDATED = "org_updated_date"
    ORG_RDAP_REF = "org_rdap_ref"

    ABUSE_HANDLE = "abuse_handle"
    ABUSE_NAME = "abuse_name"
    ABUSE_PHONE = "abuse_phone"
    ABUSE_EMAIL = "abuse_email"
    ABUSE_ADDRESS = "abuse_address"
    ABUSE_RDAP_REF = "abuse_rdap_ref"

    ROUTING_HANDLE = "routing_handle"
    ROUTING_NAME = "routing_name"
    ROUTING_PHONE = "routing_phone"
    ROUTING_EMAIL = "routing_email"
    ROUTING_ADDRESS = "routing_address"
    ROUTING_RDAP_REF = "routing_rdap_ref"

    TECH_HANDLE = "tech_handle"
    TECH_NAME = "tech_name"
    TECH_PHONE = "tech_phone"
    TECH_ADDRESS = "tech_address"
    TECH_EMAIL = "tech_email"

    def __repr__(self):
        return self.value

    def __str__(self):
        return self.value


def convert_whodap_keys(parser_output: dict) -> dict:
    # keys in both, but with mismatched names
    conversions = [
        (TLDBaseKeys.EXPIRES, "expires_date", False),
        (TLDBaseKeys.UPDATED, "updated_date", False),
        (TLDBaseKeys.CREATED, "created_date", False),
        (TLDBaseKeys.NAME_SERVERS, "nameservers", False),
        (TLDBaseKeys.REGISTRAR_ABUSE_EMAIL, "abuse_email", False),
        (TLDBaseKeys.REGISTRAR_ABUSE_PHONE, "abuse_phone", False),
        (TLDBaseKeys.TECH_EMAIL, "technical_email", False),
        (TLDBaseKeys.TECH_ADDRESS, "technical_address", False),
        (TLDBaseKeys.TECH_ORGANIZATION, "technical_organization", False),
        (TLDBaseKeys.TECH_NAME, "technical_name", False),
        (TLDBaseKeys.TECH_PHONE, "technical_phone", False),
        (TLDBaseKeys.TECH_FAX, "technical_fax", False),
        (TLDBaseKeys.REGISTRAR, "registrar_name", False),
    ]
    for asyncwhois_key, whodap_key, keep in conversions:
        if keep:
            parser_output[asyncwhois_key] = parser_output.get(whodap_key)
        else:
            parser_output[asyncwhois_key] = parser_output.pop(whodap_key)
    # asyncwhois keys not in whodap
    non_whodap_keys = [
        TLDBaseKeys.ADMIN_ID,
        TLDBaseKeys.ADMIN_CITY,
        TLDBaseKeys.ADMIN_STATE,
        TLDBaseKeys.ADMIN_COUNTRY,
        TLDBaseKeys.ADMIN_ZIPCODE,
        TLDBaseKeys.BILLING_ID,
        TLDBaseKeys.BILLING_CITY,
        TLDBaseKeys.BILLING_STATE,
        TLDBaseKeys.BILLING_COUNTRY,
        TLDBaseKeys.BILLING_ZIPCODE,
        TLDBaseKeys.TECH_ID,
        TLDBaseKeys.TECH_CITY,
        TLDBaseKeys.TECH_STATE,
        TLDBaseKeys.TECH_COUNTRY,
        TLDBaseKeys.TECH_ZIPCODE,
        TLDBaseKeys.REGISTRANT_CITY,
        TLDBaseKeys.REGISTRANT_STATE,
        TLDBaseKeys.REGISTRANT_COUNTRY,
        TLDBaseKeys.REGISTRANT_ZIPCODE,
        TLDBaseKeys.REGISTRAR_IANA_ID,
        TLDBaseKeys.REGISTRAR_URL,
        TLDBaseKeys.DNSSEC,
    ]
    for key in non_whodap_keys:
        parser_output[key] = None
    # whodap keys not in asyncwhois
    non_asyncwhois_keys = [
        "registrar_email",
        "registrar_phone",
        "registrar_address",
        "registrar_fax",
    ]
    for key in non_asyncwhois_keys:
        parser_output.pop(key)
    return parser_output


class BaseParser:
    reg_expressions = {}

    date_keys = ()
    multiple_match_keys = ()

    # For handling special cases in TLD parser classes
    known_date_formats = []
    # Extra formats that dateutil might not figure out
    extra_date_formats = [
        "%Y-%m-%dT%H:%M:%SZ[%Z]",  # 2007-01-26T19:10:31Z[UTC]
        "%Y-%m-%dT%H:%M:%S.%fZ",  # 2018-12-01T16:17:30.568Z
        "%Y-%m-%dT%H:%M:%S%zZ",  # 1970-01-01T02:00:00+02:00Z
        "%Y-%m-%dt%H:%M:%S.%fz",  # 2007-01-26t19:10:31.00z
        "%Y-%m-%d %H:%M:%SZ",  # 2000-08-22 18:55:20Z
        "before %b-%Y",  # before aug-1996
    ]
    # Additional timezone info for dateutil
    timezone_info = {
        "KST": tz.gettz("Asia/Seoul"),  # Korea Standard Time UTC+9
        "JST": tz.gettz("Asia/Tokyo"),  # Japan Standard Time UTC+9
        "EEST": tz.gettz("Europe/Athens"),  # Eastern European Summertime UTC+3
    }

    def update_reg_expressions(self, expressions_update: Dict[str, Any]) -> None:
        """
        Updates the `reg_expressions` dictionary
        :param expressions_update: dict of keys/regexes to update
        """
        self.reg_expressions.update(expressions_update)

    def parse(self, blob: str) -> Dict[Union[IPBaseKeys, TLDBaseKeys], Any]:
        """
        Iterates over the `reg_expressions` dictionary attempting to use each regex to extract values
        from `blob`, the output from the whois server.

        Assumes that the keys and regular expressions are formatted in the output `blob` such
        that a re.findall operation will work correctly. If this is not the case, you should implement
        your own version of this function in the appropriate BaseParser child class.

        :param blob: the output from the whois server
        :return: dictionary of parsed key/value pairs
        """
        parsed_output = {}
        for key, regex in self.reg_expressions.items():
            if not regex:
                parsed_output[key] = None
            else:
                many = key in self.multiple_match_keys
                parsed_output[key] = self.find_match(regex, blob, many=many)
                if key in self.date_keys and parsed_output.get(key, None):
                    parsed_output[key] = self._parse_date(parsed_output.get(key))
        return parsed_output

    def find_match(
        self,
        regex: str,
        blob: str,
        flags: re.RegexFlag = re.IGNORECASE,
        many: bool = False,
    ) -> Union[str, List[str], None]:
        """
        Performs the given regex operation on the raw output `blob`

        :param regex: the regex to use against blob
        :param blob: the raw output from the whois server
        :param flags: the optional flags to pass to the `re` method
        :param many: if True this function will use re.findall for many matches else re.search for single match
        """
        if many:
            matches = re.findall(regex, blob, flags=flags)
            return [self._process(m) for m in matches if m]
        else:
            match = re.search(regex, blob, flags=flags)
            if match:
                return self._process(match.group(1))
            return None

    def find_multiline_match(self, start: str, blob: str) -> List[str]:
        """
        Used to find multiple lines related to a single key within the
        WHOIS query response. Assumes the values are on a newline below
        the key, and that a blank line separates the last value from the
        next key or end of the text.

        Example:
        -------
        example_blob = '''
            Name servers:
                 ns1.google.com
                 ns2.google.com
                 ns3.google.com
                 ns4.google.com

        '''
        find_multiline_match('Name servers:\n', example_blob)
        # would return... ['ns1.google.com', 'ns2.google.com', 'ns3.google.com', 'ns4.google.com']

        :param start: a key that identifies where to begin the multiline search in blob
        :param blob: the whois query text
        :return: a list of values as strings
        """
        matches = []
        regex_string = start + r"\s+([A-Za-z0-9\.\s]+\n\n)"
        multiline_match = re.search(regex_string, blob, re.DOTALL | re.IGNORECASE)
        if multiline_match:
            matches = self._process_many(multiline_match.group(1))
        return matches

    def _parse_date(self, date_string: str) -> Union[datetime, str]:
        """
        Attempts to convert the given date string to a datetime.datetime object
        otherwise returns the input `date_string`
        :param date_string: a date string
        :return: a datetime.datetime object
        """

        def _datetime_or_none(dt_string: str, dt_format: str) -> Union[datetime, None]:
            try:
                return datetime.strptime(dt_string, dt_format)
            except ValueError:
                return None

        # first, try the known formats
        for date_format in self.known_date_formats:
            if date := _datetime_or_none(date_string, date_format):
                return date
        # next, try dateutil.parse
        try:
            clean_date_string = re.sub(r"\(([^)]+)\)", r"\1", date_string).strip()
            return parse(clean_date_string, tzinfos=self.timezone_info)
        except ParserError:
            pass
        # finally, try extra formats
        for date_format in self.extra_date_formats:
            if date := _datetime_or_none(date_string, date_format):
                return date
        # no luck parsing
        return date_string

    def _process_many(self, match: str) -> List[str]:
        if "\n" in match:
            match = match.split("\n")
            matches = [self._process(m) for m in match if m]
            return [m for m in matches if m]  # remove empty strings
        else:
            match = self._process(match)
            return [match] if match else []  # remove empty strings

    @staticmethod
    def _process(match: str) -> str:
        if match:
            return match.rstrip("\r").rstrip("\n").lstrip("\t").lstrip().rstrip()



# Extracted from the IANA Root Zone Database: https://www.iana.org/domains/root/db
# on 2024:09:20 12:05:03 UTC by https://github.com/pogzyb/asyncwhois/tools/tld-server-updates.py


class CountryCodeTLD:
    AC = "whois.nic.ac"
    AD = "whois.nic.ad"
    AE = "whois.aeda.net.ae"
    AF = "whois.nic.af"
    AG = "whois.nic.ag"
    AI = "whois.nic.ai"
    AL = None
    AM = "whois.amnic.net"
    AN = None
    AO = None
    AQ = None
    AR = "whois.nic.ar"
    AS = "whois.nic.as"
    AT = "whois.nic.at"
    AU = "whois.auda.org.au"
    AW = "whois.nic.aw"
    AX = "whois.ax"
    AZ = None
    BA = None
    BB = None
    BD = None
    BE = "whois.dns.be"
    BF = "whois.registre.bf"
    BG = "whois.register.bg"
    BH = "whois.nic.bh"
    BI = "whois1.nic.bi"
    BL = None
    BJ = "whois.nic.bj"
    BM = "whois.nic.bm"
    BN = "whois.bnnic.bn"
    BO = "whois.nic.bo"
    BQ = None
    BR = "whois.registro.br"
    BS = None
    BT = None
    BW = "whois.nic.net.bw"
    BY = "whois.cctld.by"
    BZ = None
    BV = None
    CA = "whois.cira.ca"
    CC = "ccwhois.verisign-grs.com"
    CD = None
    CF = "whois.dot.cf"
    CG = None
    CH = "whois.nic.ch"
    CI = "whois.nic.ci"
    CK = None
    CL = "whois.nic.cl"
    CM = None
    CN = "whois.cnnic.cn"
    CO = "whois.nic.co"
    CR = "whois.nic.cr"
    CU = None
    CV = "whois.nic.cv"
    CX = "whois.nic.cx"
    CW = None
    CY = None
    CZ = "whois.nic.cz"
    DE = "whois.denic.de"
    DJ = None
    DK = "whois.punktum.dk"
    DM = "whois.dmdomains.dm"
    DO = "whois.nic.do"
    DZ = "whois.nic.dz"
    EC = "whois.nic.ec"
    EE = "whois.tld.ee"
    EG = None
    EH = None
    ER = None
    ES = "whois.nic.es"
    ET = None
    EU = "whois.eu"
    FI = "whois.fi"
    FJ = "www.whois.fj"
    FK = None
    FM = "whois.nic.fm"
    FO = "whois.nic.fo"
    FR = "whois.nic.fr"
    GA = None
    GB = None
    GD = "whois.nic.gd"
    GE = "whois.nic.ge"
    GG = "whois.gg"
    GF = "whois.mediaserv.net"
    GH = "whois.nic.gh"
    GI = "whois.identitydigital.services"
    GL = "whois.nic.gl"
    GM = None
    GN = "whois.ande.gov.gn"
    GP = "whois.nic.gp"
    GQ = "whois.dominio.gq"
    GR = None
    GS = "whois.nic.gs"
    GT = None
    GU = None
    GW = None
    GY = "whois.registry.gy"
    HK = "whois.hkirc.hk"
    HM = "whois.registry.hm"
    HN = "whois.nic.hn"
    HR = "whois.dns.hr"
    HT = "whois.nic.ht"
    HU = "whois.nic.hu"
    ID = "whois.id"
    IE = "whois.weare.ie"
    IL = "whois.isoc.org.il"
    IM = "whois.nic.im"
    IN = "whois.registry.in"
    IO = "whois.nic.io"
    IR = "whois.nic.ir"
    IQ = "whois.cmc.iq"
    IS = "whois.isnic.is"
    IT = "whois.nic.it"
    JE = "whois.je"
    JM = None
    JO = None
    JP = "whois.jprs.jp"
    KE = "whois.kenic.or.ke"
    KG = "whois.kg"
    KH = None
    KI = "whois.nic.ki"
    KM = None
    KN = "whois.nic.kn"
    KP = None
    KR = "whois.kr"
    KW = None
    KY = "whois.kyregistry.ky"
    KZ = "whois.nic.kz"
    LA = "whois.nic.la"
    LB = "whois.lbdr.org.lb"
    LC = None
    LI = "whois.nic.li"
    LK = None
    LR = None
    LS = "whois.nic.ls"
    LT = "whois.domreg.lt"
    LU = "whois.dns.lu"
    LV = "whois.nic.lv"
    LY = "whois.nic.ly"
    MA = "whois.registre.ma"
    MC = None
    MD = "whois.nic.md"
    ME = "whois.nic.me"
    MF = None
    MG = "whois.nic.mg"
    MH = None
    MK = "whois.marnet.mk"
    ML = "whois.nic.ml"
    MM = "whois.registry.gov.mm"
    MN = "whois.nic.mn"
    MO = "whois.monic.mo"
    MQ = "whois.mediaserv.net"
    MR = "whois.nic.mr"
    MS = "whois.nic.ms"
    MP = None
    MT = None
    MU = "whois.nic.mu"
    MV = None
    MW = "whois.nic.mw"
    MY = "whois.mynic.my"
    MZ = "whois.nic.mz"
    NA = "whois.na-nic.com.na"
    MX = "whois.mx"
    NC = "whois.nc"
    NE = None
    NF = "whois.nic.nf"
    NG = "whois.nic.net.ng"
    NI = None
    NL = "whois.domain-registry.nl"
    NO = "whois.norid.no"
    NP = None
    NR = None
    NU = "whois.iis.nu"
    NZ = "whois.irs.net.nz"
    OM = "whois.registry.om"
    PA = None
    PE = "kero.yachay.pe"
    PF = "whois.registry.pf"
    PG = None
    PH = None
    PK = "whois.pknic.net.pk"
    PL = "whois.dns.pl"
    PN = None
    PM = "whois.nic.pm"
    PR = "whois.afilias-srs.net"
    PT = "whois.dns.pt"
    PS = None
    PW = "whois.nic.pw"
    PY = None
    QA = "whois.registry.qa"
    RE = "whois.nic.re"
    RO = "whois.rotld.ro"
    RS = "whois.rnids.rs"
    RU = "whois.tcinet.ru"
    RW = "whois.ricta.org.rw"
    SA = "whois.nic.net.sa"
    SB = "whois.nic.net.sb"
    SC = "whois.nic.sc"
    SD = "whois.nic.sd"
    SE = "whois.iis.se"
    SH = "whois.nic.sh"
    SG = "whois.sgnic.sg"
    SI = "whois.register.si"
    SJ = None
    SK = "whois.sk-nic.sk"
    SL = None
    SN = "whois.nic.sn"
    SM = "whois.nic.sm"
    SO = "whois.nic.so"
    SR = None
    SS = "whois.nic.ss"
    ST = "whois.nic.st"
    SU = "whois.tcinet.ru"
    SV = None
    SX = "whois.sx"
    SY = "whois.tld.sy"
    SZ = None
    TC = "whois.nic.tc"
    TD = "whois.nic.td"
    TF = "whois.nic.tf"
    TG = "whois.nic.tg"
    TH = "whois.thnic.co.th"
    TJ = None
    TK = "whois.dot.tk"
    TL = "whois.nic.tl"
    TM = "whois.nic.tm"
    TN = "whois.ati.tn"
    TO = "whois.tonic.to"
    TR = "whois.trabis.gov.tr"
    TP = None
    TT = None
    TV = "whois.nic.tv"
    TW = "whois.twnic.net.tw"
    TZ = "whois.tznic.or.tz"
    UA = "whois.ua"
    UG = "whois.co.ug"
    UK = "whois.nic.uk"
    UM = None
    UY = "whois.nic.org.uy"
    UZ = "whois.cctld.uz"
    US = "whois.nic.us"
    VA = None
    VE = "whois.nic.ve"
    VC = "whois.identitydigital.services"
    VG = "whois.nic.vg"
    VI = "virgil.nic.vi"
    VN = None
    VU = "whois.dnrs.vu"
    WF = "whois.nic.wf"
    WS = "whois.website.ws"
    XN__2SCRJ9C = "whois.registry.in"
    XN__3E0B707E = "whois.kr"
    XN__3HCRJ9C = "whois.registry.in"
    XN__45BR5CYL = "whois.registry.in"
    XN__45BRJ9C = "whois.registry.in"
    XN__4DBRK0CE = "whois.isoc.org.il"
    XN__54B7FTA0CC = None
    XN__80AO21A = "whois.nic.kz"
    XN__90A3AC = "whois.rnids.rs"
    XN__90AE = "whois.imena.bg"
    XN__90AIS = "whois.cctld.by"
    XN__CLCHC0EA0B2G2A9GCD = "whois.ta.sgnic.sg"
    XN__D1ALF = "whois.marnet.mk"
    XN__E1A4C = "whois.eu"
    XN__FIQS8S = "cwhois.cnnic.cn"
    XN__FIQZ9S = "cwhois.cnnic.cn"
    XN__FPCRJ9C3D = "whois.registry.in"
    XN__FZC2C9E2C = None
    XN__GECRJ9C = "whois.registry.in"
    XN__H2BRJ9C = "whois.registry.in"
    XN__H2BRJ9C8C = "whois.registry.in"
    XN__H2BREG3EVE = "whois.registry.in"
    XN__J1AMH = "whois.dotukr.com"
    XN__J6W193G = "whois.hkirc.hk"
    XN__KPRY57D = "whois.twnic.net.tw"
    XN__KPRW13D = "whois.twnic.net.tw"
    XN__L1ACC = None
    XN__LGBBAT1AD8J = "whois.nic.dz"
    XN__MGB9AWBF = "whois.registry.om"
    XN__MGBA3A4F16A = "whois.nic.ir"
    XN__MGBAH1A3HJKRD = "whois.nic.mr"
    XN__MGBAAM7A8H = "whois.aeda.net.ae"
    XN__MGBAI9AZGQP6J = None
    XN__MGBAYH7GPA = None
    XN__MGBBH1A71E = "whois.registry.in"
    XN__MGBBH1A = "whois.registry.in"
    XN__MGBC0A9AZCG = None
    XN__MGBCPQ6GPA1A = None
    XN__MGBERP4A5D4AR = "whois.nic.net.sa"
    XN__MGBGU82A = "whois.registry.in"
    XN__MGBPL2FH = None
    XN__MGBTX2B = "whois.cmc.iq"
    XN__MGBX4CD0AB = "whois.mynic.my"
    XN__MIX891F = "whois.monic.mo"
    XN__NODE = None
    XN__O3CW4H = "whois.thnic.co.th"
    XN__OGBPF8FL = "whois.tld.sy"
    XN__P1AI = "whois.tcinet.ru"
    XN__PGBS0DH = "whois.ati.tn"
    XN__Q7CE6A = "whois.nic.la"
    XN__QXAM = None
    XN__QXA6A = "whois.eu"
    XN__RVC1E0AM3E = "whois.registry.in"
    XN__S9BRJ9C = "whois.registry.in"
    XN__WGBH1C = None
    XN__WGBL6A = "whois.registry.qa"
    XN__XKC2AL3HYE2A = None
    XN__XKC2DL3A5EE0H = "whois.registry.in"
    XN__YFRO4I67O = "whois.zh.sgnic.sg"
    XN__Y9A3AQ = "whois.amnic.net"
    XN__YGBI2AMMX = "whois.pnina.ps"
    YE = "whois.y.net.ye"
    YT = "whois.nic.yt"
    ZA = None
    ZM = "whois.zicta.zm"
    ZW = None


class GenericTLD:
    AARP = "whois.nic.aarp"
    AAA = None
    ABB = "whois.nic.abb"
    ABBOTT = "whois.nic.abbott"
    ABARTH = None
    ABC = "whois.nic.abc"
    ABBVIE = "whois.nic.abbvie"
    ABLE = None
    ABOGADO = "whois.nic.abogado"
    ABUDHABI = "whois.nic.abudhabi"
    ACADEMY = "whois.nic.academy"
    ACCOUNTANT = "whois.nic.accountant"
    ACCENTURE = "whois.nic.accenture"
    ACCOUNTANTS = "whois.nic.accountants"
    ACO = "whois.nic.aco"
    ACTIVE = None
    ACTOR = "whois.nic.actor"
    ADS = "whois.nic.google"
    ADAC = None
    ADULT = "whois.nic.adult"
    AEG = "whois.nic.aeg"
    AETNA = None
    AFAMILYCOMPANY = None
    AFRICA = "whois.nic.africa"
    AFL = "whois.nic.afl"
    AGAKHAN = "whois.nic.agakhan"
    AGENCY = "whois.nic.agency"
    AIG = None
    AIGO = None
    AIRBUS = "whois.nic.airbus"
    AIRTEL = "whois.nic.airtel"
    AIRFORCE = "whois.nic.airforce"
    AKDN = "whois.nic.akdn"
    ALIBABA = "whois.nic.alibaba"
    ALIPAY = "whois.nic.alipay"
    ALLFINANZ = "whois.nic.allfinanz"
    ALLSTATE = "whois.nic.allstate"
    ALLY = "whois.nic.ally"
    ALFAROMEO = None
    ALSACE = "whois.nic.alsace"
    ALSTOM = "whois.nic.alstom"
    AMAZON = "whois.nic.amazon"
    AMERICANEXPRESS = None
    AMERICANFAMILY = "whois.nic.americanfamily"
    AMEX = None
    AMFAM = "whois.nic.amfam"
    AMICA = None
    AMSTERDAM = "whois.nic.amsterdam"
    ANALYTICS = None
    ANDROID = "whois.nic.google"
    ANZ = "whois.nic.anz"
    ANQUAN = "whois.teleinfo.cn"
    APARTMENTS = "whois.nic.apartments"
    APP = "whois.nic.google"
    AOL = "whois.nic.aol"
    AQUARELLE = "whois.nic.aquarelle"
    APPLE = "whois.nic.apple"
    ARAB = "whois.nic.arab"
    ARAMCO = None
    ARMY = "whois.nic.army"
    ARCHI = "whois.nic.archi"
    ART = "whois.nic.art"
    ARTE = "whois.nic.arte"
    ASDA = "whois.nic.asda"
    ASSOCIATES = "whois.nic.associates"
    ATTORNEY = "whois.nic.attorney"
    ATHLETA = None
    AUCTION = "whois.nic.auction"
    AUDIBLE = "whois.nic.audible"
    AUDIO = "whois.nic.audio"
    AUDI = "whois.nic.audi"
    AUSPOST = "whois.nic.auspost"
    AUTO = "whois.nic.auto"
    AUTOS = "whois.nic.autos"
    AUTHOR = "whois.nic.author"
    AVIANCA = None
    AWS = "whois.nic.aws"
    AXA = None
    AZURE = "whois.nic.azure"
    BABY = "whois.nic.baby"
    BAIDU = "whois.gtld.knet.cn"
    BANAMEX = None
    BAND = "whois.nic.band"
    BANK = "whois.nic.bank"
    BANANAREPUBLIC = None
    BAR = "whois.nic.bar"
    BARCLAYCARD = "whois.nic.barclaycard"
    BARCLAYS = "whois.nic.barclays"
    BARCELONA = "whois.nic.barcelona"
    BARGAINS = "whois.nic.bargains"
    BAREFOOT = "whois.nic.barefoot"
    BASEBALL = "whois.nic.baseball"
    BAUHAUS = "whois.nic.bauhaus"
    BASKETBALL = "whois.nic.basketball"
    BAYERN = "whois.nic.bayern"
    BBC = "whois.nic.bbc"
    BBT = "whois.nic.bbt"
    BBVA = "whois.nic.bbva"
    BCG = "whois.nic.bcg"
    BCN = "whois.nic.bcn"
    BEATS = "whois.nic.beats"
    BEAUTY = "whois.nic.beauty"
    BEER = "whois.nic.beer"
    BENTLEY = "whois.nic.bentley"
    BERLIN = "whois.nic.berlin"
    BESTBUY = "whois.nic.bestbuy"
    BEST = "whois.nic.best"
    BET = "whois.nic.bet"
    BHARTI = None
    BIBLE = "whois.nic.bible"
    BID = "whois.nic.bid"
    BIKE = "whois.nic.bike"
    BING = "whois.nic.bing"
    BINGO = "whois.nic.bingo"
    BIO = "whois.nic.bio"
    BLACKFRIDAY = "whois.nic.blackfriday"
    BLANCO = None
    BLACK = "whois.nic.black"
    BLOG = "whois.nic.blog"
    BLOCKBUSTER = "whois.nic.blockbuster"
    BLUE = "whois.nic.blue"
    BLOOMBERG = "whois.nic.bloomberg"
    BMW = "whois.nic.bmw"
    BNL = None
    BMS = "whois.nic.bms"
    BOATS = "whois.nic.boats"
    BOEHRINGER = "whois.nic.boehringer"
    BNPPARIBAS = "whois.nic.bnpparibas"
    BOFA = "whois.nic.bofa"
    BOM = "whois.gtlds.nic.br"
    BOND = "whois.nic.bond"
    BOO = "whois.nic.google"
    BOOK = "whois.nic.book"
    BOOKING = None
    BOOTS = None
    BOSCH = "whois.nic.bosch"
    BOSTIK = "whois.nic.bostik"
    BOT = "whois.nic.bot"
    BOSTON = "whois.nic.boston"
    BOUTIQUE = "whois.nic.boutique"
    BOX = "whois.nic.box"
    BRADESCO = "whois.nic.bradesco"
    BROADWAY = "whois.nic.broadway"
    BRIDGESTONE = "whois.nic.bridgestone"
    BROKER = "whois.nic.broker"
    BROTHER = "whois.nic.brother"
    BRUSSELS = "whois.nic.brussels"
    BUDAPEST = None
    BUGATTI = None
    BUILD = "whois.nic.build"
    BUILDERS = "whois.nic.builders"
    BUSINESS = "whois.nic.business"
    BUY = "whois.nic.buy"
    BUZZ = "whois.nic.buzz"
    BZH = "whois.nic.bzh"
    CAB = "whois.nic.cab"
    CAFE = "whois.nic.cafe"
    CAL = "whois.nic.google"
    CALL = "whois.nic.call"
    CALVINKLEIN = None
    CAM = "whois.nic.cam"
    CAMERA = "whois.nic.camera"
    CAMP = "whois.nic.camp"
    CANCERRESEARCH = None
    CAPETOWN = "whois.nic.capetown"
    CANON = "whois.nic.canon"
    CAPITALONE = "whois.nic.capitalone"
    CAR = "whois.nic.car"
    CAPITAL = "whois.nic.capital"
    CARDS = "whois.nic.cards"
    CARAVAN = None
    CARE = "whois.nic.care"
    CAREERS = "whois.nic.careers"
    CAREER = "whois.nic.career"
    CARTIER = None
    CARS = "whois.nic.cars"
    CASE = "whois.nic.case"
    CASA = "whois.nic.casa"
    CASEIH = None
    CASINO = "whois.nic.casino"
    CATERING = "whois.nic.catering"
    CASH = "whois.nic.cash"
    CBA = "whois.nic.cba"
    CBN = None
    CBRE = None
    CATHOLIC = "whois.nic.catholic"
    CBS = None
    CEB = None
    CENTER = "whois.nic.center"
    CERN = "whois.nic.cern"
    CFA = "whois.nic.cfa"
    CEO = "whois.nic.ceo"
    CFD = "whois.nic.cfd"
    CHANEL = "whois.nic.chanel"
    CHARITY = "whois.nic.charity"
    CHASE = None
    CHANNEL = "whois.nic.google"
    CHAT = "whois.nic.chat"
    CHEAP = "whois.nic.cheap"
    CHINTAI = "whois.nic.chintai"
    CHRISTMAS = "whois.nic.christmas"
    CHROME = "whois.nic.google"
    CHLOE = None
    CHURCH = "whois.nic.church"
    CIPRIANI = "whois.nic.cipriani"
    CHRYSLER = None
    CIRCLE = "whois.nic.circle"
    CISCO = None
    CITADEL = "whois.nic.citadel"
    CITI = None
    CITIC = None
    CITY = "whois.nic.city"
    CITYEATS = None
    CLAIMS = "whois.nic.claims"
    CLICK = "whois.nic.click"
    CLEANING = "whois.nic.cleaning"
    CLINIC = "whois.nic.clinic"
    CLINIQUE = "whois.nic.clinique"
    CLOUD = "whois.nic.cloud"
    CLOTHING = "whois.nic.clothing"
    CLUB = "whois.nic.club"
    CLUBMED = "whois.nic.clubmed"
    COACH = "whois.nic.coach"
    CODES = "whois.nic.codes"
    COLLEGE = "whois.nic.college"
    COLOGNE = "whois.ryce-rsp.com"
    COM = "whois.verisign-grs.com"
    COFFEE = "whois.nic.coffee"
    COMCAST = None
    COMMBANK = "whois.nic.commbank"
    COMMUNITY = "whois.nic.community"
    COMPANY = "whois.nic.company"
    COMPARE = "whois.nic.compare"
    COMPUTER = "whois.nic.computer"
    COMSEC = "whois.nic.comsec"
    CONDOS = "whois.nic.condos"
    CONSTRUCTION = "whois.nic.construction"
    CONTACT = "whois.nic.contact"
    CONSULTING = "whois.nic.consulting"
    CONTRACTORS = "whois.nic.contractors"
    COOKING = "whois.nic.cooking"
    COOL = "whois.nic.cool"
    COOKINGCHANNEL = None
    CORSICA = "whois.nic.corsica"
    COUNTRY = "whois.nic.country"
    COUPONS = "whois.nic.coupons"
    COUPON = None
    COURSES = "whois.nic.courses"
    CPA = "whois.nic.cpa"
    CREDIT = "whois.nic.credit"
    CREDITUNION = "whois.nic.creditunion"
    CRICKET = "whois.nic.cricket"
    CREDITCARD = "whois.nic.creditcard"
    CROWN = "whois.nic.crown"
    CRUISE = "whois.nic.cruise"
    CRS = "whois.nic.crs"
    CRUISES = "whois.nic.cruises"
    CSC = None
    CUISINELLA = "whois.nic.cuisinella"
    CYMRU = "whois.nic.cymru"
    CYOU = "whois.nic.cyou"
    DABUR = "whois.nic.dabur"
    DAD = "whois.nic.google"
    DANCE = "whois.nic.dance"
    DATA = "whois.nic.data"
    DATE = "whois.nic.date"
    DATING = "whois.nic.dating"
    DATSUN = "whois.nic.gmo"
    DAY = "whois.nic.google"
    DCLK = "whois.nic.google"
    DDS = "whois.nic.dds"
    DEALER = "whois.nic.dealer"
    DEAL = "whois.nic.deal"
    DEGREE = "whois.nic.degree"
    DEALS = "whois.nic.deals"
    DELL = None
    DELIVERY = "whois.nic.delivery"
    DELTA = "whois.nic.delta"
    DEMOCRAT = "whois.nic.democrat"
    DENTAL = "whois.nic.dental"
    DELOITTE = "whois.nic.deloitte"
    DESI = "whois.nic.desi"
    DENTIST = "whois.nic.dentist"
    DEV = "whois.nic.google"
    DESIGN = "whois.nic.design"
    DIAMONDS = "whois.nic.diamonds"
    DIET = "whois.nic.diet"
    DHL = None
    DIRECT = "whois.nic.direct"
    DIRECTORY = "whois.nic.directory"
    DIGITAL = "whois.nic.digital"
    DISCOVER = None
    DISH = "whois.nic.dish"
    DISCOUNT = "whois.nic.discount"
    DIY = "whois.nic.diy"
    DOCS = "whois.nic.google"
    DOCTOR = "whois.nic.doctor"
    DNP = "whois.nic.dnp"
    DOG = "whois.nic.dog"
    DODGE = None
    DOHA = None
    DOMAINS = "whois.nic.domains"
    DOT = "whois.nic.dot"
    DOWNLOAD = "whois.nic.download"
    DOOSAN = None
    DRIVE = "whois.nic.google"
    DUBAI = "whois.nic.dubai"
    DTV = "whois.nic.dtv"
    DUNLOP = "whois.nic.dunlop"
    DUCK = None
    DUNS = None
    DUPONT = None
    DURBAN = "whois.nic.durban"
    DVAG = "whois.nic.dvag"
    DVR = "whois.nic.dvr"
    EARTH = "whois.nic.earth"
    EAT = "whois.nic.google"
    ECO = "whois.nic.eco"
    EDUCATION = "whois.nic.education"
    EDEKA = "whois.nic.edeka"
    EMERCK = "whois.nic.emerck"
    EMAIL = "whois.nic.email"
    ENGINEER = "whois.nic.engineer"
    ENGINEERING = "whois.nic.engineering"
    ENERGY = "whois.nic.energy"
    ENTERPRISES = "whois.nic.enterprises"
    EPOST = None
    EPSON = "whois.nic.epson"
    ERICSSON = "whois.nic.ericsson"
    EQUIPMENT = "whois.nic.equipment"
    ERNI = "whois.nic.erni"
    ESQ = "whois.nic.google"
    ESTATE = "whois.nic.estate"
    ESURANCE = None
    EUROVISION = "whois.nic.eurovision"
    ETISALAT = None
    EVENTS = "whois.nic.events"
    EUS = "whois.nic.eus"
    EVERBANK = None
    EXCHANGE = "whois.nic.exchange"
    EXPERT = "whois.nic.expert"
    EXPRESS = "whois.nic.express"
    EXTRASPACE = "whois.nic.extraspace"
    EXPOSED = "whois.nic.exposed"
    FAGE = "whois.nic.fage"
    FAIL = "whois.nic.fail"
    FAITH = "whois.nic.faith"
    FAIRWINDS = "whois.nic.fairwinds"
    FAN = "whois.nic.fan"
    FANS = "whois.nic.fans"
    FAMILY = "whois.nic.family"
    FARMERS = None
    FASHION = "whois.nic.fashion"
    FARM = "whois.nic.farm"
    FEDEX = "whois.nic.fedex"
    FEEDBACK = "whois.nic.feedback"
    FAST = "whois.nic.fast"
    FERRARI = "whois.nic.ferrari"
    FIAT = None
    FERRERO = None
    FIDELITY = "whois.nic.fidelity"
    FIDO = "whois.nic.fido"
    FINAL = "whois.gtlds.nic.br"
    FINANCE = "whois.nic.finance"
    FILM = "whois.nic.film"
    FINANCIAL = "whois.nic.financial"
    FIRESTONE = "whois.nic.firestone"
    FIRE = "whois.nic.fire"
    FISH = "whois.nic.fish"
    FIRMDALE = "whois.nic.firmdale"
    FIT = "whois.nic.fit"
    FISHING = "whois.nic.fishing"
    FLICKR = None
    FITNESS = "whois.nic.fitness"
    FLIR = None
    FLORIST = "whois.nic.florist"
    FLIGHTS = "whois.nic.flights"
    FLOWERS = "whois.nic.flowers"
    FLY = "whois.nic.google"
    FLSMIDTH = None
    FOOD = None
    FOODNETWORK = None
    FOOTBALL = "whois.nic.football"
    FORD = None
    FOO = "whois.nic.google"
    FOREX = "whois.nic.forex"
    FORSALE = "whois.nic.forsale"
    FORUM = "whois.nic.forum"
    FOUNDATION = "whois.nic.foundation"
    FOX = "whois.nic.fox"
    FREE = "whois.nic.free"
    FRL = "whois.nic.frl"
    FROGANS = "whois.nic.frogans"
    FRONTDOOR = None
    FRESENIUS = "whois.nic.fresenius"
    FTR = None
    FRONTIER = None
    FUJITSU = "whois.nic.gmo"
    FUJIXEROX = None
    FUN = "whois.nic.fun"
    FUND = "whois.nic.fund"
    FURNITURE = "whois.nic.furniture"
    FUTBOL = "whois.nic.futbol"
    FYI = "whois.nic.fyi"
    GALLERY = "whois.nic.gallery"
    GAL = "whois.nic.gal"
    GALLO = "whois.nic.gallo"
    GAME = "whois.nic.game"
    GAMES = "whois.nic.games"
    GALLUP = "whois.nic.gallup"
    GAP = None
    GARDEN = "whois.nic.garden"
    GAY = "whois.nic.gay"
    GBIZ = "whois.nic.google"
    GDN = "whois.nic.gdn"
    GENT = "whois.nic.gent"
    GENTING = "whois.nic.genting"
    GEA = "whois.nic.gea"
    GEORGE = "whois.nic.george"
    GGEE = "whois.nic.ggee"
    GIFT = "whois.uniregistry.net"
    GIFTS = "whois.nic.gifts"
    GIVES = "whois.nic.gives"
    GIVING = "whois.nic.giving"
    GLASS = "whois.nic.glass"
    GLADE = None
    GLE = "whois.nic.google"
    GLOBAL = "whois.nic.global"
    GLOBO = "whois.gtlds.nic.br"
    GMAIL = "whois.nic.google"
    GMBH = "whois.nic.gmbh"
    GMX = "whois.nic.gmx"
    GMO = "whois.nic.gmo"
    GODADDY = "whois.nic.godaddy"
    GOLD = "whois.nic.gold"
    GOLDPOINT = "whois.nic.goldpoint"
    GOLF = "whois.nic.golf"
    GOO = "whois.nic.gmo"
    GOODYEAR = "whois.nic.goodyear"
    GOOG = "whois.nic.google"
    GOODHANDS = None
    GOOGLE = "whois.nic.google"
    GOP = "whois.nic.gop"
    GOT = "whois.nic.got"
    GRAINGER = None
    GRAPHICS = "whois.nic.graphics"
    GRATIS = "whois.nic.gratis"
    GREEN = "whois.nic.green"
    GRIPE = "whois.nic.gripe"
    GROCERY = "whois.nic.grocery"
    GROUP = "whois.nic.group"
    GUARDIAN = None
    GUCCI = "whois.nic.gucci"
    GUGE = "whois.nic.google"
    GUIDE = "whois.nic.guide"
    GUITARS = "whois.nic.guitars"
    GURU = "whois.nic.guru"
    HAIR = "whois.nic.hair"
    HAMBURG = "whois.nic.hamburg"
    HANGOUT = "whois.nic.google"
    HAUS = "whois.nic.haus"
    HDFC = "whois.nic.hdfc"
    HDFCBANK = "whois.nic.hdfcbank"
    HEALTH = None
    HBO = None
    HEALTHCARE = "whois.nic.healthcare"
    HELP = "whois.nic.help"
    HERE = "whois.nic.google"
    HERMES = "whois.nic.hermes"
    HGTV = None
    HELSINKI = "whois.nic.helsinki"
    HIPHOP = "whois.nic.hiphop"
    HISAMITSU = "whois.nic.gmo"
    HIV = "whois.nic.hiv"
    HITACHI = "whois.nic.gmo"
    HKT = "whois.nic.hkt"
    HOCKEY = "whois.nic.hockey"
    HOLIDAY = "whois.nic.holiday"
    HOMEDEPOT = "whois.nic.homedepot"
    HOLDINGS = "whois.nic.holdings"
    HOMEGOODS = None
    HOMES = "whois.nic.homes"
    HONDA = "whois.nic.honda"
    HONEYWELL = None
    HOMESENSE = None
    HORSE = "whois.nic.horse"
    HOSPITAL = "whois.nic.hospital"
    HOST = "whois.nic.host"
    HOT = "whois.nic.hot"
    HOTELES = None
    HOTELS = "whois.nic.hotels"
    HOSTING = "whois.nic.hosting"
    HOTMAIL = "whois.nic.hotmail"
    HOW = "whois.nic.google"
    HSBC = None
    HOUSE = "whois.nic.house"
    HTC = None
    HUGHES = "whois.nic.hughes"
    HYATT = None
    HYUNDAI = "whois.nic.hyundai"
    IBM = "whois.nic.ibm"
    ICBC = "whois.nic.icbc"
    ICU = "whois.nic.icu"
    ICE = "whois.nic.ice"
    IFM = "whois.nic.ifm"
    IEEE = None
    IKANO = "whois.nic.ikano"
    IINET = None
    IMDB = "whois.nic.imdb"
    IMAMAT = "whois.nic.imamat"
    IMMO = "whois.nic.immo"
    IMMOBILIEN = "whois.nic.immobilien"
    INDUSTRIES = "whois.nic.industries"
    INFINITI = "whois.nic.gmo"
    INC = "whois.nic.inc"
    INFO = "whois.nic.info"
    INK = "whois.nic.ink"
    INSTITUTE = "whois.nic.institute"
    ING = "whois.nic.google"
    INSURANCE = "whois.nic.insurance"
    INSURE = "whois.nic.insure"
    INTEL = None
    INTUIT = None
    INVESTMENTS = "whois.nic.investments"
    INTERNATIONAL = "whois.nic.international"
    IPIRANGA = None
    ISELECT = None
    IRISH = "whois.nic.irish"
    ISMAILI = "whois.nic.ismaili"
    IST = "whois.nic.ist"
    ISTANBUL = "whois.nic.istanbul"
    ITAU = None
    ITV = "whois.nic.itv"
    IWC = None
    JAGUAR = "whois.nic.jaguar"
    JAVA = "whois.nic.java"
    IVECO = None
    JCP = None
    JCB = "whois.nic.gmo"
    JEEP = "whois.nic.jeep"
    JETZT = "whois.nic.jetzt"
    JEWELRY = "whois.nic.jewelry"
    JIO = "whois.nic.jio"
    JLC = None
    JLL = "whois.nic.jll"
    JMP = None
    JNJ = None
    JOBURG = "whois.nic.joburg"
    JOT = "whois.nic.jot"
    JOY = "whois.nic.joy"
    JPMORGAN = None
    JPRS = None
    JUEGOS = "whois.uniregistry.net"
    JUNIPER = "whois.nic.juniper"
    KAUFEN = "whois.nic.kaufen"
    KERRYHOTELS = "whois.nic.kerryhotels"
    KDDI = "whois.nic.kddi"
    KERRYLOGISTICS = "whois.nic.kerrylogistics"
    KFH = "whois.nic.kfh"
    KERRYPROPERTIES = "whois.nic.kerryproperties"
    KIA = "whois.nic.kia"
    KIDS = "whois.nic.kids"
    KIM = "whois.nic.kim"
    KINDLE = "whois.nic.kindle"
    KITCHEN = "whois.nic.kitchen"
    KIWI = "whois.nic.kiwi"
    KINDER = None
    KOELN = "whois.ryce-rsp.com"
    KOMATSU = "whois.nic.komatsu"
    KOSHER = "whois.nic.kosher"
    KPN = None
    KPMG = None
    KRD = "whois.nic.krd"
    KRED = None
    KUOKGROUP = "whois.nic.kuokgroup"
    KYOTO = "whois.nic.kyoto"
    LACAIXA = "whois.nic.lacaixa"
    LADBROKES = None
    LAMBORGHINI = "whois.nic.lamborghini"
    LAMER = "whois.nic.lamer"
    LANCASTER = "whois.nic.lancaster"
    LANCIA = None
    LAND = "whois.nic.land"
    LANCOME = None
    LANDROVER = "whois.nic.landrover"
    LANXESS = None
    LASALLE = "whois.nic.lasalle"
    LAT = "whois.nic.lat"
    LATROBE = "whois.nic.latrobe"
    LATINO = "whois.nic.latino"
    LAWYER = "whois.nic.lawyer"
    LDS = "whois.nic.lds"
    LAW = "whois.nic.law"
    LEASE = "whois.nic.lease"
    LECLERC = "whois.nic.leclerc"
    LEFRAK = "whois.nic.lefrak"
    LEGAL = "whois.nic.legal"
    LEGO = "whois.nic.lego"
    LEXUS = "whois.nic.lexus"
    LGBT = "whois.nic.lgbt"
    LIAISON = None
    LIDL = "whois.nic.lidl"
    LIFE = "whois.nic.life"
    LIFEINSURANCE = "whois.nic.lifeinsurance"
    LIFESTYLE = "whois.nic.lifestyle"
    LIKE = "whois.nic.like"
    LIGHTING = "whois.nic.lighting"
    LILLY = None
    LIMO = "whois.nic.limo"
    LIMITED = "whois.nic.limited"
    LINCOLN = None
    LINDE = None
    LIPSY = "whois.nic.lipsy"
    LINK = "whois.uniregistry.net"
    LIVE = "whois.nic.live"
    LIVING = None
    LIXIL = None
    LLC = "whois.nic.llc"
    LLP = "whois.nic.llp"
    LOAN = "whois.nic.loan"
    LOANS = "whois.nic.loans"
    LOCKER = "whois.nic.locker"
    LOCUS = "whois.nic.locus"
    LOFT = None
    LOL = "whois.nic.lol"
    LONDON = "whois.nic.london"
    LOTTE = "whois.nic.lotte"
    LOVE = "whois.nic.love"
    LPL = "whois.nic.lpl"
    LPLFINANCIAL = "whois.nic.lplfinancial"
    LOTTO = "whois.nic.lotto"
    LTD = "whois.nic.ltd"
    LTDA = "whois.nic.ltda"
    LUNDBECK = "whois.nic.lundbeck"
    LUPIN = None
    LUXE = "whois.nic.luxe"
    LUXURY = "whois.nic.luxury"
    MACYS = None
    MADRID = "whois.nic.madrid"
    MAIF = "whois.nic.maif"
    MAKEUP = "whois.nic.makeup"
    MAN = "whois.nic.man"
    MAISON = "whois.nic.maison"
    MANGO = "whois.nic.mango"
    MAP = "whois.nic.google"
    MANAGEMENT = "whois.nic.management"
    MARKET = "whois.nic.market"
    MARKETING = "whois.nic.marketing"
    MARRIOTT = "whois.nic.marriott"
    MARSHALLS = None
    MASERATI = None
    MARKETS = "whois.nic.markets"
    MATTEL = None
    MBA = "whois.nic.mba"
    MCD = None
    MCDONALDS = None
    MCKINSEY = "whois.nic.mckinsey"
    MED = "whois.nic.med"
    MEET = "whois.nic.google"
    MEDIA = "whois.nic.media"
    MEME = "whois.nic.google"
    MELBOURNE = "whois.nic.melbourne"
    MEN = "whois.nic.men"
    MEMORIAL = "whois.nic.memorial"
    MEO = None
    MERCKMSD = "whois.nic.merckmsd"
    MENU = "whois.nic.menu"
    METLIFE = None
    MICROSOFT = "whois.nic.microsoft"
    MIAMI = "whois.nic.miami"
    MINI = "whois.nic.mini"
    MINT = None
    MITSUBISHI = "whois.nic.gmo"
    MIT = "whois.nic.mit"
    MLB = None
    MLS = "whois.nic.mls"
    MMA = "whois.nic.mma"
    MOBI = "whois.nic.mobi"
    MOBILE = "whois.nic.mobile"
    MODA = "whois.nic.moda"
    MOBILY = None
    MOI = "whois.nic.moi"
    MOE = "whois.nic.moe"
    MOM = "whois.nic.mom"
    MONASH = "whois.nic.monash"
    MONEY = "whois.nic.money"
    MONSTER = "whois.nic.monster"
    MONTBLANC = None
    MOPAR = None
    MORTGAGE = "whois.nic.mortgage"
    MORMON = "whois.nic.mormon"
    MOSCOW = "whois.nic.moscow"
    MOTO = "whois.nic.moto"
    MOTORCYCLES = "whois.nic.motorcycles"
    MOV = "whois.nic.google"
    MOVIE = "whois.nic.movie"
    MOVISTAR = None
    MSD = "whois.nic.msd"
    MTN = "whois.nic.mtn"
    MTPC = None
    MTR = "whois.nic.mtr"
    MUSIC = "whois.nic.music"
    MUTUAL = None
    MUTUELLE = None
    NAB = "whois.nic.nab"
    NADEX = None
    NAGOYA = "whois.nic.nagoya"
    NATIONWIDE = None
    NATURA = None
    NAVY = "whois.nic.navy"
    NBA = None
    NEC = "whois.nic.nec"
    NET = "whois.verisign-grs.com"
    NETBANK = "whois.nic.netbank"
    NETFLIX = None
    NETWORK = "whois.nic.network"
    NEW = "whois.nic.google"
    NEUSTAR = None
    NEWHOLLAND = None
    NEXT = "whois.nic.next"
    NEXTDIRECT = "whois.nic.nextdirect"
    NEWS = "whois.nic.news"
    NEXUS = "whois.nic.google"
    NGO = "whois.nic.ngo"
    NHK = "whois.nic.nhk"
    NFL = None
    NICO = "whois.nic.nico"
    NIKE = None
    NINJA = "whois.nic.ninja"
    NIKON = "whois.nic.nikon"
    NISSAN = "whois.nic.gmo"
    NISSAY = "whois.nic.nissay"
    NOKIA = "whois.nic.nokia"
    NORTHWESTERNMUTUAL = None
    NORTON = "whois.nic.norton"
    NOW = "whois.nic.now"
    NOWTV = "whois.nic.nowtv"
    NOWRUZ = "whois.nic.nowruz"
    NRA = "whois.nic.nra"
    NRW = "whois.nic.nrw"
    NYC = "whois.nic.nyc"
    NTT = None
    OBI = "whois.nic.obi"
    OBSERVER = "whois.nic.observer"
    OFF = None
    OFFICE = "whois.nic.office"
    OKINAWA = "whois.nic.okinawa"
    OLAYAN = "whois.nic.olayan"
    OLAYANGROUP = "whois.nic.olayangroup"
    OLDNAVY = None
    OLLO = "whois.nic.ollo"
    OMEGA = "whois.nic.omega"
    ONE = "whois.nic.one"
    ONG = "whois.nic.ong"
    ONL = "whois.nic.onl"
    ONYOURSIDE = None
    ONLINE = "whois.nic.online"
    OPEN = "whois.nic.open"
    OOO = "whois.nic.ooo"
    ORACLE = "whois.nic.oracle"
    ORG = "whois.publicinterestregistry.org"
    ORGANIC = "whois.nic.organic"
    ORIENTEXPRESS = None
    ORANGE = "whois.nic.orange"
    OSAKA = "whois.nic.osaka"
    ORIGINS = "whois.nic.origins"
    OTSUKA = "whois.nic.otsuka"
    OTT = "whois.nic.ott"
    OVH = "whois.nic.ovh"
    PAGE = "whois.nic.google"
    PAMPEREDCHEF = None
    PANASONIC = "whois.nic.gmo"
    PANERAI = None
    PARS = "whois.nic.pars"
    PARTNERS = "whois.nic.partners"
    PARIS = "whois.nic.paris"
    PARTS = "whois.nic.parts"
    PARTY = "whois.nic.party"
    PAY = "whois.nic.pay"
    PCCW = "whois.nic.pccw"
    PASSAGENS = None
    PET = "whois.nic.pet"
    PFIZER = None
    PHARMACY = "whois.nic.pharmacy"
    PHD = "whois.nic.google"
    PHILIPS = "whois.nic.philips"
    PHOTO = "whois.nic.photo"
    PHOTOGRAPHY = "whois.nic.photography"
    PHONE = "whois.nic.phone"
    PHOTOS = "whois.nic.photos"
    PHYSIO = "whois.nic.physio"
    PIAGET = None
    PICS = "whois.nic.pics"
    PICTURES = "whois.nic.pictures"
    PID = "whois.nic.pid"
    PICTET = "whois.nic.pictet"
    PIN = "whois.nic.pin"
    PING = "whois.nic.ping"
    PINK = "whois.nic.pink"
    PIZZA = "whois.nic.pizza"
    PIONEER = "whois.nic.pioneer"
    PLACE = "whois.nic.place"
    PLAY = "whois.nic.google"
    PLAYSTATION = "whois.nic.playstation"
    PLUMBING = "whois.nic.plumbing"
    PLUS = "whois.nic.plus"
    PNC = "whois.nic.pnc"
    POHL = "whois.nic.pohl"
    POKER = "whois.nic.poker"
    POLITIE = "whois.nic.politie"
    PORN = "whois.nic.porn"
    PRAXI = None
    PRAMERICA = None
    PRESS = "whois.nic.press"
    PRIME = "whois.nic.prime"
    PROD = "whois.nic.google"
    PRODUCTIONS = "whois.nic.productions"
    PROF = "whois.nic.google"
    PROGRESSIVE = "whois.nic.progressive"
    PROMO = "whois.nic.promo"
    PROPERTIES = "whois.nic.properties"
    PROPERTY = "whois.nic.property"
    PROTECTION = "whois.nic.protection"
    PRU = None
    PRUDENTIAL = None
    PUB = "whois.nic.pub"
    PWC = "whois.nic.pwc"
    QPON = "whois.nic.qpon"
    QUEBEC = "whois.nic.quebec"
    QUEST = "whois.nic.quest"
    RACING = "whois.nic.racing"
    QVC = None
    RADIO = "whois.nic.radio"
    RAID = None
    READ = "whois.nic.read"
    REALESTATE = "whois.nic.realestate"
    REALTOR = "whois.nic.realtor"
    REALTY = "whois.nic.realty"
    RECIPES = "whois.nic.recipes"
    RED = "whois.nic.red"
    REDSTONE = "whois.nic.redstone"
    REDUMBRELLA = "whois.nic.redumbrella"
    REHAB = "whois.nic.rehab"
    REISE = "whois.nic.reise"
    REISEN = "whois.nic.reisen"
    RELIANCE = "whois.nic.reliance"
    REIT = "whois.nic.reit"
    REN = "whois.nic.ren"
    RENT = "whois.nic.rent"
    RENTALS = "whois.nic.rentals"
    REPAIR = "whois.nic.repair"
    REPUBLICAN = "whois.nic.republican"
    REPORT = "whois.nic.report"
    REST = "whois.nic.rest"
    RESTAURANT = "whois.nic.restaurant"
    REVIEW = "whois.nic.review"
    REXROTH = "whois.nic.rexroth"
    REVIEWS = "whois.nic.reviews"
    RICH = "whois.nic.rich"
    RICOH = "whois.nic.ricoh"
    RICHARDLI = "whois.nic.richardli"
    RIL = "whois.nic.ril"
    RIO = "whois.gtlds.nic.br"
    RIGHTATHOME = None
    RMIT = None
    RIP = "whois.nic.rip"
    ROCKS = "whois.nic.rocks"
    ROCHER = None
    RODEO = "whois.nic.rodeo"
    ROGERS = "whois.nic.rogers"
    ROOM = "whois.nic.room"
    RSVP = "whois.nic.google"
    RUHR = "whois.nic.ruhr"
    RUGBY = "whois.nic.rugby"
    RUN = "whois.nic.run"
    RWE = "whois.nic.rwe"
    RYUKYU = "whois.nic.ryukyu"
    SAARLAND = "whois.nic.saarland"
    SAFETY = "whois.nic.safety"
    SAFE = "whois.nic.safe"
    SALE = "whois.nic.sale"
    SALON = "whois.nic.salon"
    SAKURA = None
    SAMSCLUB = "whois.nic.samsclub"
    SAMSUNG = "whois.nic.samsung"
    SANDVIK = "whois.nic.sandvik"
    SANDVIKCOROMANT = "whois.nic.sandvikcoromant"
    SAP = "whois.nic.sap"
    SAPO = None
    SANOFI = "whois.nic.sanofi"
    SARL = "whois.nic.sarl"
    SAVE = "whois.nic.save"
    SAXO = "whois.nic.saxo"
    SAS = None
    SBS = "whois.nic.sbs"
    SBI = "whois.nic.sbi"
    SCA = None
    SCB = "whois.nic.scb"
    SCHAEFFLER = "whois.afilias-srs.net"
    SCHMIDT = "whois.nic.schmidt"
    SCHOLARSHIPS = "whois.nic.scholarships"
    SCHOOL = "whois.nic.school"
    SCHULE = "whois.nic.schule"
    SCHWARZ = "whois.nic.schwarz"
    SCIENCE = "whois.nic.science"
    SCJOHNSON = None
    SCOT = "whois.nic.scot"
    SCOR = None
    SEARCH = "whois.nic.google"
    SEAT = "whois.nic.seat"
    SECURE = "whois.nic.secure"
    SEEK = "whois.nic.seek"
    SELECT = "whois.nic.select"
    SECURITY = "whois.nic.security"
    SENER = "whois.nic.rwe"
    SES = None
    SEVEN = "whois.nic.seven"
    SERVICES = "whois.nic.services"
    SEX = "whois.nic.sex"
    SEW = "whois.nic.sew"
    SFR = "whois.nic.sfr"
    SEXY = "whois.nic.sexy"
    SHARP = "whois.nic.gmo"
    SHANGRILA = "whois.nic.shangrila"
    SHAW = None
    SHIA = "whois.nic.shia"
    SHELL = "whois.nic.shell"
    SHOES = "whois.nic.shoes"
    SHIKSHA = "whois.nic.shiksha"
    SHOP = "whois.nic.shop"
    SHOPPING = "whois.nic.shopping"
    SHOUJI = "whois.teleinfo.cn"
    SHOW = "whois.nic.show"
    SHRIRAM = None
    SHOWTIME = None
    SINA = "whois.nic.sina"
    SILK = "whois.nic.silk"
    SINGLES = "whois.nic.singles"
    SITE = "whois.nic.site"
    SKI = "whois.nic.ski"
    SKIN = "whois.nic.skin"
    SKY = "whois.nic.sky"
    SKYPE = "whois.nic.skype"
    SLING = "whois.nic.sling"
    SMART = "whois.nic.smart"
    SMILE = "whois.nic.smile"
    SNCF = "whois.nic.sncf"
    SOCCER = "whois.nic.soccer"
    SOCIAL = "whois.nic.social"
    SOFTWARE = "whois.nic.software"
    SOFTBANK = "whois.nic.softbank"
    SOHU = None
    SOLUTIONS = "whois.nic.solutions"
    SONG = None
    SOLAR = "whois.nic.solar"
    SONY = "whois.nic.sony"
    SOY = "whois.nic.google"
    SPA = "whois.nic.spa"
    SPACE = "whois.nic.space"
    SPIEGEL = None
    SPOT = "whois.nic.spot"
    SPREADBETTING = None
    SPORT = "whois.nic.sport"
    SRL = "whois.nic.srl"
    SRT = None
    STAPLES = None
    STAR = "whois.nic.star"
    STADA = "whois.nic.stada"
    STATEBANK = "whois.nic.statebank"
    STATEFARM = None
    STARHUB = None
    STATOIL = None
    STCGROUP = "whois.nic.stcgroup"
    STOCKHOLM = "whois.nic.stockholm"
    STC = "whois.nic.stc"
    STORAGE = "whois.nic.storage"
    STREAM = "whois.nic.stream"
    STUDIO = "whois.nic.studio"
    STORE = "whois.nic.store"
    STUDY = "whois.nic.study"
    STYLE = "whois.nic.style"
    SUCKS = "whois.nic.sucks"
    SUPPLY = "whois.nic.supply"
    SUPPLIES = "whois.nic.supplies"
    SUPPORT = "whois.nic.support"
    SURF = "whois.nic.surf"
    SURGERY = "whois.nic.surgery"
    SUZUKI = "whois.nic.suzuki"
    SWATCH = "whois.nic.swatch"
    SWIFTCOVER = None
    SWISS = "whois.nic.swiss"
    SYDNEY = "whois.nic.sydney"
    SYMANTEC = None
    SYSTEMS = "whois.nic.systems"
    TAB = "whois.nic.tab"
    TALK = "whois.nic.talk"
    TAOBAO = "whois.nic.taobao"
    TAIPEI = "whois.nic.taipei"
    TARGET = None
    TATAMOTORS = "whois.nic.tatamotors"
    TATAR = "whois.nic.tatar"
    TATTOO = "whois.nic.tattoo"
    TAX = "whois.nic.tax"
    TAXI = "whois.nic.taxi"
    TCI = "whois.nic.tci"
    TDK = "whois.nic.tdk"
    TEAM = "whois.nic.team"
    TECHNOLOGY = "whois.nic.technology"
    TECH = "whois.nic.tech"
    TELECITY = None
    TEMASEK = "whois.nic.temasek"
    TELEFONICA = None
    TEVA = "whois.nic.teva"
    TENNIS = "whois.nic.tennis"
    THD = "whois.nic.thd"
    THEATRE = "whois.nic.theatre"
    THEATER = "whois.nic.theater"
    TIAA = "whois.nic.tiaa"
    TICKETS = "whois.nic.tickets"
    TIFFANY = None
    TIPS = "whois.nic.tips"
    TIRES = "whois.nic.tires"
    TIENDA = "whois.nic.tienda"
    TIROL = "whois.nic.tirol"
    TJMAXX = None
    TJX = None
    TKMAXX = None
    TMALL = "whois.nic.tmall"
    TODAY = "whois.nic.today"
    TOKYO = "whois.nic.tokyo"
    TOOLS = "whois.nic.tools"
    TOP = "whois.nic.top"
    TORAY = "whois.nic.toray"
    TOTAL = "whois.nic.total"
    TOURS = "whois.nic.tours"
    TOWN = "whois.nic.town"
    TOSHIBA = "whois.nic.toshiba"
    TOYOTA = "whois.nic.toyota"
    TOYS = "whois.nic.toys"
    TRADE = "whois.nic.trade"
    TRADING = "whois.nic.trading"
    TRAINING = "whois.nic.training"
    TRAVELCHANNEL = None
    TRAVELERS = "whois.nic.travelers"
    TRAVELERSINSURANCE = "whois.nic.travelersinsurance"
    TRV = "whois.nic.trv"
    TRUST = "whois.nic.trust"
    TUBE = "whois.nic.tube"
    TUI = "whois.nic.tui"
    TUNES = "whois.nic.tunes"
    TUSHU = "whois.nic.tushu"
    TVS = "whois.nic.tvs"
    UBANK = "whois.nic.ubank"
    UCONNECT = None
    UNICOM = "whois.nic.unicom"
    UNIVERSITY = "whois.nic.university"
    UNO = "whois.nic.uno"
    UBS = "whois.nic.ubs"
    UOL = "whois.gtlds.nic.br"
    UPS = "whois.nic.ups"
    VANA = "whois.nic.vana"
    VACATIONS = "whois.nic.vacations"
    VANGUARD = "whois.nic.vanguard"
    VEGAS = "whois.nic.vegas"
    VENTURES = "whois.nic.ventures"
    VERISIGN = "whois.nic.verisign"
    VERSICHERUNG = "whois.nic.versicherung"
    VET = "whois.nic.vet"
    VIAJES = "whois.nic.viajes"
    VIDEO = "whois.nic.video"
    VIKING = "whois.nic.viking"
    VILLAS = "whois.nic.villas"
    VIG = "whois.nic.vig"
    VIP = "whois.nic.vip"
    VIRGIN = "whois.nic.virgin"
    VIN = "whois.nic.vin"
    VISION = "whois.nic.vision"
    VISA = "whois.nic.visa"
    VISTAPRINT = None
    VISTA = None
    VIVO = None
    VIVA = "whois.nic.viva"
    VLAANDEREN = "whois.nic.vlaanderen"
    VOLKSWAGEN = None
    VODKA = "whois.nic.vodka"
    VOLVO = "whois.nic.volvo"
    VOTE = "whois.nic.vote"
    VOTING = "whois.nic.voting"
    VOTO = "whois.nic.voto"
    VOYAGE = "whois.nic.voyage"
    VUELOS = None
    WALES = "whois.nic.wales"
    WALMART = "whois.nic.walmart"
    WALTER = "whois.nic.walter"
    WANG = "whois.gtld.knet.cn"
    WANGGOU = "whois.nic.wanggou"
    WARMAN = None
    WATCH = "whois.nic.watch"
    WEATHER = None
    WATCHES = "whois.nic.watches"
    WEATHERCHANNEL = None
    WEBCAM = "whois.nic.webcam"
    WEBSITE = "whois.nic.website"
    WEBER = "whois.nic.weber"
    WED = "whois.nic.wed"
    WEIBO = "whois.nic.weibo"
    WEDDING = "whois.nic.wedding"
    WEIR = "whois.nic.weir"
    WIEN = "whois.nic.wien"
    WHOSWHO = "whois.nic.whoswho"
    WIKI = "whois.nic.wiki"
    WIN = "whois.nic.win"
    WILLIAMHILL = None
    WINE = "whois.nic.wine"
    WINNERS = None
    WINDOWS = "whois.nic.windows"
    WOLTERSKLUWER = "whois.nic.wolterskluwer"
    WOODSIDE = "whois.nic.woodside"
    WORK = "whois.nic.work"
    WORKS = "whois.nic.works"
    WORLD = "whois.nic.world"
    WOW = "whois.nic.wow"
    WME = "whois.nic.wme"
    WTF = "whois.nic.wtf"
    XBOX = "whois.nic.xbox"
    WTC = "whois.nic.wtc"
    XEROX = "whois.nic.xerox"
    XFINITY = None
    XIHUAN = "whois.teleinfo.cn"
    XIN = "whois.nic.xin"
    XN__11B4C3D = "whois.nic.xn--11b4c3d"
    XN__1QQW23A = "whois.ngtld.cn"
    XN__1CK2E1B = None
    XN__30RR7Y = "whois.gtld.knet.cn"
    XN__3DS443G = "whois.teleinfo.cn"
    XN__3BST00M = "whois.gtld.knet.cn"
    XN__3PXU8K = "whois.nic.xn--3pxu8k"
    XN__42C2D9A = "whois.nic.xn--42c2d9a"
    XN__3OQ18VL8PN36A = None
    XN__45Q11C = "whois.gtld.knet.cn"
    XN__4GBRIM = "whois.nic.xn--4gbrim"
    XN__55QX5D = "whois.ngtld.cn"
    XN__5SU34J936BGSG = "whois.nic.xn--5su34j936bgsg"
    XN__5TZM5G = "whois.nic.xn--5tzm5g"
    XN__55QW42G = "whois.conac.cn"
    XN__6FRZ82G = "whois.nic.xn--6frz82g"
    XN__6QQ986B3XL = "whois.gtld.knet.cn"
    XN__80ADXHKS = "whois.nic.xn--80adxhks"
    XN__80AQECDR1A = "whois.nic.xn--80aqecdr1a"
    XN__80ASEHDB = "whois.nic.xn--80asehdb"
    XN__80ASWG = "whois.nic.xn--80aswg"
    XN__8Y0A063A = "whois.nic.xn--8y0a063a"
    XN__9DBQ2A = "whois.nic.xn--9dbq2a"
    XN__9KRT00A = "whois.nic.xn--9krt00a"
    XN__9ET52U = "whois.gtld.knet.cn"
    XN__B4W605FERD = "whois.nic.xn--b4w605ferd"
    XN__C1AVG = "whois.nic.xn--c1avg"
    XN__BCK1B9A5DRE4C = None
    XN__C2BR7G = "whois.nic.xn--c2br7g"
    XN__CCK2B3B = None
    XN__CCKWCXETD = "whois.nic.xn--cckwcxetd"
    XN__CZR694B = None
    XN__CG4BKI = "whois.kr"
    XN__CZRS0T = "whois.nic.xn--czrs0t"
    XN__D1ACJ3B = "whois.nic.xn--d1acj3b"
    XN__CZRU2D = "whois.gtld.knet.cn"
    XN__ECKVDTC9D = None
    XN__EFVY88H = "whois.nic.xn--efvy88h"
    XN__ESTV75G = None
    XN__FCT429K = None
    XN__FIQ228C5HS = "whois.teleinfo.cn"
    XN__FHBEI = "whois.nic.xn--fhbei"
    XN__FIQ64B = "whois.gtld.knet.cn"
    XN__FJQ720A = "whois.nic.xn--fjq720a"
    XN__FLW351E = "whois.nic.google"
    XN__FZYS8D69UVGM = "whois.nic.xn--fzys8d69uvgm"
    XN__G2XX48C = None
    XN__GCKR3F0F = None
    XN__GK3AT1E = None
    XN__HXT814E = "whois.gtld.knet.cn"
    XN__I1B6B1A6A2E = "whois.nic.xn--i1b6b1a6a2e"
    XN__IMR513N = None
    XN__IO0A7I = "whois.ngtld.cn"
    XN__JLQ480N2RG = "whois.nic.xn--jlq480n2rg"
    XN__J1AEF = "whois.nic.xn--j1aef"
    XN__JLQ61U9W7B = None
    XN__JVR189M = None
    XN__KCRX77D1X4A = "whois.nic.xn--kcrx77d1x4a"
    XN__KPU716F = None
    XN__KPUT3I = "whois.nic.xn--kput3i"
    XN__MGBA3A3EJT = None
    XN__MGBA7C0BBN0A = "whois.nic.xn--mgba7c0bbn0a"
    XN__MGBAAKC7DVF = None
    XN__MGBAB2BD = "whois.nic.xn--mgbab2bd"
    XN__MGBB9FBPOB = None
    XN__MGBCA7DZDO = "whois.nic.xn--mgbca7dzdo"
    XN__MGBI4ECEXP = "whois.nic.xn--mgbi4ecexp"
    XN__MGBT3DHD = "whois.nic.xn--mgbt3dhd"
    XN__MK1BU44C = "whois.nic.xn--mk1bu44c"
    XN__MXTQ1M = "whois.nic.xn--mxtq1m"
    XN__NGBE9E0A = "whois.nic.xn--ngbe9e0a"
    XN__NGBRX = "whois.nic.xn--ngbrx"
    XN__NGBC5AZD = "whois.nic.xn--ngbc5azd"
    XN__NQV7FS00EMA = "whois.nic.xn--nqv7fs00ema"
    XN__NQV7F = "whois.nic.xn--nqv7f"
    XN__NYQY26A = None
    XN__OTU796D = None
    XN__P1ACF = "whois.nic.xn--p1acf"
    XN__PBT977C = None
    XN__PSSY2U = "whois.nic.xn--pssy2u"
    XN__Q9JYB4C = "whois.nic.google"
    XN__QCKA1PMC = "whois.nic.google"
    XN__RHQV96G = None
    XN__ROVU88B = None
    XN__SES554G = "whois.nic.xn--ses554g"
    XN__T60B56A = "whois.nic.xn--t60b56a"
    XN__TCKWE = "whois.nic.xn--tckwe"
    XN__TIQ49XQYJ = "whois.nic.xn--tiq49xqyj"
    XN__UNUP4Y = "whois.nic.xn--unup4y"
    XN__VERMGENSBERATER_CTB = "whois.nic.xn--vermgensberater-ctb"
    XN__VHQUV = "whois.nic.xn--vhquv"
    XN__VERMGENSBERATUNG_PWB = "whois.nic.xn--vermgensberatung-pwb"
    XN__W4R85EL8FHU5DNRA = "whois.nic.xn--w4r85el8fhu5dnra"
    XN__VUQ861B = "whois.teleinfo.cn"
    XN__W4RS40L = "whois.nic.xn--w4rs40l"
    XN__XHQ521B = "whois.ngtld.cn"
    XPERIA = None
    XN__ZFR164B = "whois.conac.cn"
    XYZ = "whois.nic.xyz"
    YAHOO = "whois.nic.yahoo"
    YAMAXUN = "whois.nic.yamaxun"
    YANDEX = None
    YACHTS = "whois.nic.yachts"
    YODOBASHI = "whois.nic.gmo"
    YOGA = "whois.nic.yoga"
    YOKOHAMA = "whois.nic.yokohama"
    YOU = "whois.nic.you"
    YOUTUBE = "whois.nic.google"
    YUN = "whois.teleinfo.cn"
    ZARA = "whois.nic.zara"
    ZAPPOS = "whois.nic.zappos"
    ZIP = "whois.nic.google"
    ZIPPO = None
    ZERO = None
    ZONE = "whois.nic.zone"
    ZUERICH = "whois.nic.zuerich"


class SponsoredTLD:
    AERO = "whois.aero"
    ASIA = "whois.nic.asia"
    CAT = "whois.nic.cat"
    COOP = "whois.nic.coop"
    EDU = "whois.educause.edu"
    GOV = "whois.dotgov.gov"
    INT = "whois.iana.org"
    JOBS = "whois.nic.jobs"
    MIL = None
    MUSEUM = "whois.nic.museum"
    POST = "whois.dotpostregistry.net"
    TEL = "whois.nic.tel"
    TRAVEL = "whois.nic.travel"
    XXX = "whois.nic.xxx"



from typing import Tuple
from ipaddress import IPv4Address

from ..errors import GeneralError
from .ipv4 import IPV4_ALLOCATIONS, AllocationsT
from .domains import CountryCodeTLD, GenericTLD, SponsoredTLD


class IPv4Allocations:
    """Regional Internet Registry IPv4 Allocations:
    https://www.iana.org/assignments/ipv4-address-space/ipv4-address-space.xhtml
    and https://data.iana.org/rdap/ipv4.json
    """

    _allocations: AllocationsT = IPV4_ALLOCATIONS

    def get_servers(self, ipv4: IPv4Address) -> Tuple[str, str]:
        """
        Retrieves the WHOIS and RDAP servers for the given IPv4 address.
        """
        for network, servers in self._allocations.items():
            if ipv4 in network:
                return servers["rdap"], servers["whois"]
        # no match
        raise GeneralError(f"No WHOIS or RDAP server for: {ipv4}")



#!/usr/bin/env python

"""This tool can be used to update the list mapping between IPv4 blocks and
their RDAP & Whois servers.

Data is retrieved from
https://www.iana.org/assignments/ipv4-address-space/ipv4-address-space.xhtm

More specifically from
https://www.iana.org/assignments/ipv4-address-space/ipv4-address-space.csv

Then, parsed and converted to the structured used by asyncwhois.

This tool should (arguably) be piped to "black -" to enhance the readability
of the produced content and then redirected to a source file in asyncwhois.

e.g:

    $ ipv4allocs | black - > asyncwhois/servers/ipv4.py


This tool currently relies on pypi:`httpx` to perform HTTP(s) request and on
pypi:`netaddr` 3rd party module to minimize the number of entries produced in
the final structure

"""

import os.path
import sys
import asyncio
import csv
import textwrap
from datetime import datetime, timezone
from typing import NamedTuple, TextIO
from ipaddress import IPv4Network
from io import StringIO

from netaddr import IPSet
import httpx


class Servers(NamedTuple):
    """A tuple linking Matching whois and rdap servers altogether"""

    whois: tuple[str]
    rdap: tuple[str]


AllocationMapping = dict[Servers, IPSet]
"""The type of data produced by the :py:func:`parse` function."""


IPv4Allocations = dict[IPv4Network, dict[str, str]]
"""The data format used by asyncwhois"""


ALLOCATIONS_CSV_URL = (
    "https://www.iana.org/assignments/ipv4-address-space" "/ipv4-address-space.csv"
)
"""The URL of the IANA document containing of the list of whois/rdap servers
matching IPv4 blocks."""


async def retrieve_allocation_data(url: str) -> str:
    """Simply perform an HTTP GET query and return the text"""
    async with httpx.AsyncClient() as client:
        response = await client.get(url)
        return response.text


def parse(csv_file: TextIO) -> AllocationMapping:
    """From a given CSV text file, return a minimal AllocationMapping object
    (Servers -> :py:class:`netaddr.IPSet`)
    """
    servers_to_set: AllocationMapping = {}
    keys = ["WHOIS", "RDAP"]
    csv_raw = csv.DictReader(csv_file)
    for data in csv_raw:
        if any(data[key] for key in keys):
            servers = Servers(*[tuple(data[key].split("\n")) for key in keys])
            first_byte = data["Prefix"].split("/", maxsplit=1)[0].lstrip("0")
            network = f"{first_byte}.0.0.0/8"
            try:
                ipset = servers_to_set[servers]
            except KeyError:
                ipset = servers_to_set[servers] = IPSet()
            ipset.add(network)
    return servers_to_set


def compat(data: AllocationMapping) -> IPv4Allocations:
    """Given an AllocationMapping structure, return the structured currently
    used by asyncwhois (an IPv4Network -> Servers mapping)
    """
    return {
        IPv4Network(str(cidr)): {
            "rdap": servers.rdap[0],
            "whois": servers.whois[0],
        }
        for servers, ipset in data.items()
        for cidr in ipset.iter_cidrs()
    }


def create_module_file(allocations: IPv4Allocations) -> str:
    """Generate the output module content"""
    ts = datetime.now(tz=timezone.utc).strftime("%Y:%m:%d %H:%M:%S")
    progname = os.path.basename(sys.argv[0])
    mapping_type_name = "AllocationsT"
    return textwrap.dedent(
        f'''
    """
    This file has been generated by {progname} program on {ts} UTC

    If you need it updated, override this file content with {progname} output.
    """


    from ipaddress import IPv4Network


    {mapping_type_name} = dict[IPv4Network, dict[str, str]]


    IPV4_ALLOCATIONS: {mapping_type_name} = {allocations}


    __all__ = ("IPV4_ALLOCATIONS", "{mapping_type_name}")
    '''
    )


async def main() -> None:
    """Retrieve the allocation data, parse it and generate a basic data-only
    python module.
    """
    csv_content: str = await retrieve_allocation_data(ALLOCATIONS_CSV_URL)
    allocation_data = parse(StringIO(csv_content))
    print(create_module_file(compat(allocation_data)))


if __name__ == "__main__":
    asyncio.run(main())



# https://docs.aiohttp.org/en/stable/web_quickstart.html
import httpx
from aiohttp import web

import asyncwhois


async def whois_handler(request):
    domain = request.match_info.get("domain", "google.com")
    result = await asyncwhois.aio_whois_domain(domain)
    return web.Response(
        text=f"WhoIs Query Parsed:\n{result.parser_output}\nQuery Output:\n{result.query_output}"
    )


app = web.Application()
app.add_routes([web.get("/whois/{domain}", whois_handler)])
web.run_app(app)

#
# Then navigate to localhost:8080/whois/<domain>
#
# --- examples ---
# localhost:8080/whois/netflix.com
# localhost:8080/whois/google.com
# localhost:8080/whois/pypi.org
#



import time
import random

import httpx
from llama_index.core import PromptTemplate
from llama_index.llms.ollama import Ollama

import asyncwhois


def get_llm(model_name: str):
    # todo: adjust LLM params like temperature, etc.
    ollama_llm = Ollama(model_name, request_timeout=60.0)
    return ollama_llm


# todo: do prompt engineering! try few-shot examples?
PROMPT_TEMPLATE = """\
Below is the text output from a WHOIS server.
---------------------
{whois_text}
---------------------
Using the information contained in the text above and not prior knowledge, create a JSON formatted document
containing the information as keys and values. The JSON document should align with the following format where 
$VALUE is the value for the given key:
{{
    admin_address: $VALUE,
    admin_city: $VALUE,
    admin_country: $VALUE,
    admin_email: $VALUE,
    admin_fax: $VALUE,
    admin_id: $VALUE,
    admin_name: $VALUE,
    admin_organization: $VALUE,
    admin_phone: $VALUE,
    admin_state: $VALUE,
    admin_zipcode: $VALUE,
    billing_address: $VALUE,
    billing_city: $VALUE,
    billing_country: $VALUE,
    billing_email: $VALUE,
    billing_fax: $VALUE,
    billing_id: $VALUE,
    billing_name: $VALUE,
    billing_organization: $VALUE,
    billing_phone: $VALUE,
    billing_state: $VALUE,
    billing_zipcode: $VALUE,
    created: $VALUE,
    dnssec: $VALUE,
    domain_name: $VALUE,
    expires: $VALUE,
    name_servers: $VALUE,
    registrant_address: $VALUE,
    registrant_city: $VALUE,
    registrant_country: $VALUE,
    registrant_email: $VALUE,
    registrant_fax: $VALUE,
    registrant_name: $VALUE,
    registrant_organization: $VALUE,
    registrant_phone: $VALUE,
    registrant_state: $VALUE,
    registrant_zipcode: $VALUE,
    registrar: $VALUE,
    registrar_abuse_email: $VALUE,
    registrar_abuse_phone: $VALUE,
    registrar_iana_id: $VALUE,
    registrar_url: $VALUE,
    status: [$VALUE],
    tech_address: $VALUE,
    tech_city: $VALUE,
    tech_country: $VALUE,
    tech_email: $VALUE,
    tech_fax: $VALUE,
    tech_id: $VALUE,
    tech_name: $VALUE,
    tech_organization: $VALUE,
    tech_phone: $VALUE,
    tech_state: $VALUE,
    tech_zipcode: $VALUE,
    updated: $VALUE
}}
"""


if __name__ == "__main__":
    # load google supported tlds
    google_supported_domains = httpx.get(
        "https://www.google.com/supported_domains"
    ).text.split()

    # randomly chooses one and saves it
    # todo: this is just an example to get you started, please feel free to modify this
    #   to pull and save all or a subset of the domains.
    domain = random.choice(google_supported_domains)
    query_output, _ = asyncwhois.whois(domain, authoritative_only=True)
    # todo: after the first run, you can just read in this file without having to re-query the server.
    with open(f"./data/{domain}.txt", "w") as output:
        output.write(query_output)

    # get an LLM and give it the prompt
    llm = get_llm("mistral-nemo:12b")
    prompt = PromptTemplate(PROMPT_TEMPLATE).format(context_str=query_output)
    response = llm.complete(prompt)

    # save results
    with open(f"output_{domain}_{time.time_ns()}.txt", "w") as out:
        out.write(str(response))

    # todo: test the result! Feel free to manual/anecdotal or empirical methods.



import unittest
import os

from asyncwhois.parse_tld import DomainParser


class TestTLDParsers(unittest.TestCase):
    @staticmethod
    def get_txt(tld: str):
        with open(
            os.path.join(
                os.path.abspath(os.path.dirname(__file__)), f"samples/tld_{tld}.txt"
            ),
            encoding="utf-8",
        ) as txt_input:
            query_output = txt_input.read()
        return query_output

    def setUp(self):
        self.parser = DomainParser()

    def test_parser_com(self):
        query_output = self.get_txt("com")
        parser_output = self.parser.parse(query_output, "com")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 1997)
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2028)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(updated_date.month, 9)
        self.assertEqual(expires_date.month, 9)
        self.assertEqual(created_date.day, 15)
        self.assertEqual(updated_date.day, 9)
        self.assertEqual(expires_date.day, 13)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc.")
        self.assertEqual(parser_output.get("registrar_iana_id"), "292")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 6)

    def test_parser_in(self):
        query_output = self.get_txt("in")
        parser_output = self.parser.parse(query_output, "in")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2007)
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(created_date.month, 12)
        self.assertEqual(updated_date.month, 12)
        self.assertEqual(expires_date.month, 12)
        self.assertEqual(created_date.day, 1)
        self.assertEqual(updated_date.day, 1)
        self.assertEqual(expires_date.day, 1)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "Rajasthan")
        self.assertEqual(parser_output.get("registrant_country"), "IN")
        self.assertEqual(parser_output.get("registrant_address"), None)
        self.assertEqual(parser_output.get("registrant_zipcode"), None)

        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_top(self):
        query_output = self.get_txt("top")
        parser_output = self.parser.parse(query_output, "top")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2020)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 2)
        self.assertEqual(updated_date.month, 5)
        self.assertEqual(expires_date.month, 2)
        self.assertEqual(created_date.day, 25)
        self.assertEqual(updated_date.day, 22)
        self.assertEqual(expires_date.day, 25)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "AZ")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_zipcode"), "85016")
        self.assertEqual(
            parser_output.get("registrant_address"),
            "1928 E. Highland Ave. Ste F104 PMB# 255",
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "NameSilo, LLC")
        # registrant
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "See PrivacyGuardian.org",
        )
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 3)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_xyz(self):
        query_output = self.get_txt("xyz")
        parser_output = self.parser.parse(query_output, "xyz")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2019)
        self.assertEqual(updated_date.year, 1)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(updated_date.month, 1)
        self.assertEqual(expires_date.month, 10)
        self.assertEqual(created_date.day, 15)
        self.assertEqual(updated_date.day, 1)
        self.assertEqual(expires_date.day, 15)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "Panama")
        self.assertEqual(parser_output.get("registrant_country"), "PA")
        self.assertEqual(parser_output.get("registrant_zipcode"), None)
        self.assertEqual(parser_output.get("registrant_address"), "P.O. Box 0823-03411")
        # registrar
        self.assertEqual(parser_output.get("registrar"), "NAMECHEAP INC")
        # registrant
        self.assertEqual(
            parser_output.get("registrant_organization"), "WhoisGuard, Inc."
        )
        self.assertEqual(parser_output.get("registrant_name"), "WhoisGuard Protected")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 3)
        self.assertEqual(len(parser_output.get("status")), 2)

    def test_parser_ir(self):
        query_output = self.get_txt("ir")
        parser_output = self.parser.parse(query_output, "ir")
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date, None)
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(updated_date.month, 11)
        self.assertEqual(expires_date.month, 12)
        self.assertEqual(updated_date.day, 7)
        self.assertEqual(expires_date.day, 22)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), None)
        self.assertEqual(parser_output.get("registrant_country"), None)
        self.assertEqual(parser_output.get("registrant_zipcode"), None)
        self.assertEqual(
            parser_output.get("registrant_address"),
            "1600 Amphitheatre Parkway, Mountain View, CA, US",
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), None)
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google Inc.")
        self.assertEqual(parser_output.get("registrant_name"), "Google Inc.")

    def test_parser_icu(self):
        query_output = self.get_txt("icu")
        parser_output = self.parser.parse(query_output, "icu")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2019)
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(created_date.month, 5)
        self.assertEqual(updated_date.month, 10)
        self.assertEqual(expires_date.month, 5)
        self.assertEqual(created_date.day, 11)
        self.assertEqual(updated_date.day, 23)
        self.assertEqual(expires_date.day, 11)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "Sind(en)")
        self.assertEqual(parser_output.get("registrant_city"), "karachi")
        self.assertEqual(parser_output.get("registrant_country"), "PK")
        self.assertEqual(parser_output.get("registrant_zipcode"), "75640")
        self.assertEqual(parser_output.get("registrant_address"), "Manzoor Colony")
        # registrar
        self.assertEqual(
            parser_output.get("registrar"),
            "PDR Ltd. d/b/a PublicDomainRegistry.com",
        )
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), None)
        self.assertEqual(parser_output.get("registrant_name"), None)
        # misc
        self.assertEqual(parser_output.get("dnssec"), "Unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 4)

    def test_parser_ie(self):
        query_output = self.get_txt("ie")
        parser_output = self.parser.parse(query_output, "ie")
        # confirm dates
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 21)
        self.assertEqual(expires_date.day, 21)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "Markmonitor Inc")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), None)
        self.assertEqual(parser_output.get("registrant_name"), "Google, Inc")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 3)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_uk(self):
        query_output = self.get_txt("uk")
        parser_output = self.parser.parse(query_output, "uk")
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2014)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 5)
        self.assertEqual(expires_date.month, 6)
        self.assertEqual(created_date.day, 11)
        self.assertEqual(updated_date.day, 10)
        self.assertEqual(expires_date.day, 11)
        # registrar
        self.assertEqual(
            parser_output.get("registrar"),
            "Markmonitor Inc. t/a MarkMonitor Inc. [Tag = MARKMONITOR]",
        )
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), None)
        self.assertEqual(parser_output.get("registrant_name"), None)
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_cl(self):
        query_output = self.get_txt("cl")
        tld = "cl"
        parser_output = self.parser.parse(query_output, "cl")
        # confirm dates
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(expires_date.month, 11)
        self.assertEqual(created_date.day, 22)
        self.assertEqual(expires_date.day, 20)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)

    def test_parser_be(self):
        query_output = self.get_txt("be")
        tld = "be"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2000)
        self.assertEqual(created_date.month, 12)
        self.assertEqual(created_date.day, 12)
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), None)
        self.assertEqual(
            parser_output.get("registrant_name"),
            "Not shown, please visit www.dnsbelgium.be for webbased whois.",
        )

    def test_parser_de(self):
        query_output = self.get_txt("de")
        tld = "de"
        parser_output = self.parser.parse(query_output, tld)

        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parse_ua(self):
        query_output = self.get_txt("ua")
        tld = "ua"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(updated_date.year, 2022)
        self.assertEqual(expires_date.year, 2023)
        self.assertEqual(created_date.month, 12)
        self.assertEqual(updated_date.month, 11)
        self.assertEqual(expires_date.month, 12)
        self.assertEqual(created_date.day, 4)
        self.assertEqual(updated_date.day, 2)
        self.assertEqual(expires_date.day, 4)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        self.assertEqual(parser_output.get("registrar_url"), "http://markmonitor.com")
        self.assertEqual(
            parser_output.get("registrar_abuse_email"),
            "abusecomplaints@markmonitor.com",
        )
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), None)
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        # misc
        self.assertEqual(parser_output.get("dnssec"), None)
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 7)

    def test_parse_ua1(self):
        query_output = self.get_txt("ua1")
        tld = "ua"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2018)
        self.assertEqual(updated_date.year, 2021)
        self.assertEqual(expires_date.year, 2023)
        self.assertEqual(created_date.month, 11)
        self.assertEqual(updated_date.month, 10)
        self.assertEqual(expires_date.month, 10)
        self.assertEqual(created_date.day, 25)
        self.assertEqual(updated_date.day, 17)
        self.assertEqual(expires_date.day, 16)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "ua.imena")

    def test_parse_us(self):
        query_output = self.get_txt("us")
        tld = "us"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 4)
        self.assertEqual(updated_date.month, 3)
        self.assertEqual(expires_date.month, 4)
        self.assertEqual(created_date.day, 19)
        self.assertEqual(updated_date.day, 22)
        self.assertEqual(expires_date.day, 18)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_name"), "Google Inc")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 3)

    def test_parse_ar(self):
        query_output = self.get_txt("ar")
        tld = "ar"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2013)
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2020)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(updated_date.month, 11)
        self.assertEqual(expires_date.month, 11)
        self.assertEqual(created_date.day, 29)
        self.assertEqual(updated_date.day, 1)
        self.assertEqual(expires_date.day, 1)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "nicar")
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "GOOGLE INC.")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 0)

    def test_parse_no(self):
        query_output = self.get_txt("no")
        tld = "no"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertIsNone(expires_date)
        self.assertEqual(created_date.year, 2001)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(created_date.month, 2)
        self.assertEqual(updated_date.month, 1)
        self.assertEqual(created_date.day, 26)
        self.assertEqual(updated_date.day, 27)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "REG466-NORID")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)

    def test_parser_ai(self):
        query_output = self.get_txt("ai")
        tld = "ai"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2017)
        self.assertEqual(updated_date.year, 2019)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 12)
        self.assertEqual(updated_date.month, 8)
        self.assertEqual(expires_date.month, 9)
        self.assertEqual(created_date.day, 16)
        self.assertEqual(updated_date.day, 24)
        self.assertEqual(expires_date.day, 25)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "Markmonitor")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_name"), "Domain Administrator")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 3)

    def test_parser_me(self):
        query_output = self.get_txt("me")
        tld = "me"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2008)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 5)
        self.assertEqual(expires_date.month, 6)
        self.assertEqual(created_date.day, 13)
        self.assertEqual(updated_date.day, 12)
        self.assertEqual(expires_date.day, 13)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 6)

    def test_parser_cc(self):
        query_output = self.get_txt("cc")
        tld = "cc"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(updated_date.year, 2016)
        self.assertEqual(expires_date.year, 2024)
        self.assertEqual(created_date.month, 8)
        self.assertEqual(updated_date.month, 11)
        self.assertEqual(expires_date.month, 8)
        self.assertEqual(created_date.day, 4)
        self.assertEqual(updated_date.day, 12)
        self.assertEqual(expires_date.day, 4)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_city"), "Cupertino")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_zipcode"), "95014")
        self.assertEqual(parser_output.get("registrant_address"), "1 Infinite Loop")
        # registrar
        self.assertEqual(parser_output.get("registrar"), "CSC CORPORATE DOMAINS, INC.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Apple Inc.")
        self.assertEqual(parser_output.get("registrant_name"), "Domain Administrator")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_ru(self):
        query_output = self.get_txt("ru")
        tld = "ru"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2004)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 3)
        self.assertEqual(expires_date.day, 4)
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("admin_email"), "https://www.nic.ru/whois")

    def test_parser_edu(self):
        query_output = self.get_txt("edu")
        tld = "edu"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 1985)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(updated_date.month, 12)
        self.assertEqual(expires_date.month, 7)
        self.assertEqual(created_date.day, 7)
        self.assertEqual(updated_date.day, 26)
        self.assertEqual(expires_date.day, 31)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), None)
        self.assertEqual(parser_output.get("registrant_city"), None)
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_zipcode"), None)
        self.assertEqual(
            parser_output.get("registrant_address"),
            "ITCS, Arbor Lakes, 4251 Plymouth Road, Ann Arbor, MI 48105-2785",
        )
        # registrant
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "University of Michigan -- ITD",
        )
        self.assertEqual(
            parser_output.get("registrant_name"), "University of Michigan -- ITD"
        )
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 3)

    def test_parser_info(self):
        query_output = self.get_txt("info")
        tld = "info"
        parser_output = self.parser.parse(query_output, tld)
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2001)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 7)
        self.assertEqual(updated_date.month, 6)
        self.assertEqual(expires_date.month, 7)
        self.assertEqual(created_date.day, 31)
        self.assertEqual(updated_date.day, 29)
        self.assertEqual(expires_date.day, 31)
        # geo
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 6)

    def test_parser_fi(self):
        query_output = self.get_txt("fi")
        tld = "fi"
        parser_output = self.parser.parse(query_output, tld)
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2006)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 6)
        self.assertEqual(expires_date.month, 7)
        self.assertEqual(created_date.day, 30)
        self.assertEqual(updated_date.day, 2)
        self.assertEqual(expires_date.day, 4)
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        # geo
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(
            parser_output.get("registrant_country"), "United States of America"
        )
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("dnssec"), "no")

    def test_parser_kz(self):
        query_output = self.get_txt("kz")
        tld = "kz"
        parser_output = self.parser.parse(query_output, tld)
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 1999)
        self.assertEqual(updated_date.year, 2012)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 11)
        self.assertEqual(created_date.day, 7)
        self.assertEqual(updated_date.day, 28)
        # geo
        self.assertEqual(
            parser_output.get("registrant_address"), "2400 E. Bayshore Pkwy"
        )
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Google Inc.")
        self.assertEqual(parser_output.get("registrant_organization"), "Google Inc.")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("registrar"), "KAZNIC")

    def test_parser_si(self):
        query_output = self.get_txt("si")
        tld = "si"
        parser_output = self.parser.parse(query_output, tld)
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2005)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 4)
        self.assertEqual(expires_date.month, 7)
        self.assertEqual(created_date.day, 4)
        self.assertEqual(expires_date.day, 19)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "G830057")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor")

    def test_parser_ae(self):
        query_output = self.get_txt("ae")
        tld = "ae"
        parser_output = self.parser.parse(query_output, tld)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Domain Administrator")
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 2)
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor")

    def test_parser_ve(self):
        query_output = self.get_txt("ve")
        tld = "ve"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")

        # registered: 05.06.2002 20:00:00
        # expire: 06.05.2021

        self.assertEqual(created_date.year, 2002)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 4)
        self.assertEqual(expires_date.month, 5)
        self.assertEqual(created_date.day, 5)
        self.assertEqual(updated_date.day, 24)
        self.assertEqual(expires_date.day, 6)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "Ca")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")
        self.assertEqual(parser_output.get("registrant_zipcode"), "94043")
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "NIC-VE")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google Llc")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)

    def test_parser_app(self):
        query_output = self.get_txt("app")
        tld = "app"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2018)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(updated_date.month, 3)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 29)
        self.assertEqual(updated_date.day, 2)
        self.assertEqual(expires_date.day, 29)
        # geo
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_city"), "REDACTED FOR PRIVACY")
        self.assertEqual(
            parser_output.get("registrant_zipcode"), "REDACTED FOR PRIVACY"
        )
        self.assertEqual(
            parser_output.get("registrant_address"), "REDACTED FOR PRIVACY"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 3)

    def test_parser_cn(self):
        query_output = self.get_txt("cn")
        tld = "cn"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2003)
        self.assertEqual(expires_date.year, 2022)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 17)
        self.assertEqual(expires_date.day, 17)
        # geo
        self.assertEqual(
            parser_output.get("registrant_name"), "北京谷翔信息技术有限公司"
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "厦门易名科技股份有限公司")
        # registrant
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 5)
        self.assertEqual(parser_output.get("dnssec"), "unsigned")

    def test_parser_co(self):
        query_output = self.get_txt("co")
        tld = "co"
        parser_output = self.parser.parse(query_output, tld)
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2010)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 2)
        self.assertEqual(updated_date.month, 1)
        self.assertEqual(expires_date.month, 2)
        self.assertEqual(created_date.day, 25)
        self.assertEqual(updated_date.day, 28)
        self.assertEqual(expires_date.day, 24)
        # geo
        self.assertEqual(
            parser_output.get("registrant_address"), "REDACTED FOR PRIVACY"
        )
        self.assertEqual(
            parser_output.get("registrant_zipcode"), "REDACTED FOR PRIVACY"
        )
        self.assertEqual(parser_output.get("registrant_city"), "REDACTED FOR PRIVACY")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "REDACTED FOR PRIVACY")
        self.assertEqual(parser_output.get("registrant_organization"), "Google Inc.")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 3)
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc.")
        self.assertEqual(parser_output.get("dnssec"), "unsigned")

    def test_parser_pl(self):
        query_output = self.get_txt("pl")
        tld = "pl"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2002)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2023)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(updated_date.month, 8)
        self.assertEqual(expires_date.month, 10)
        self.assertEqual(created_date.day, 19)
        self.assertEqual(updated_date.day, 17)
        self.assertEqual(expires_date.day, 14)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "Markmonitor, Inc.")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "Unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)

    def test_parser_online(self):
        query_output = self.get_txt("online")
        tld = "online"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2015)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 8)
        self.assertEqual(updated_date.month, 8)
        self.assertEqual(expires_date.month, 8)
        self.assertEqual(created_date.day, 19)
        self.assertEqual(updated_date.day, 25)
        self.assertEqual(expires_date.day, 19)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc (TLDs)")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "unsigned")
        self.assertEqual(len(parser_output.get("name_servers")), 4)

    def test_parser_buzz(self):
        query_output = self.get_txt("buzz")
        tld = "buzz"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2014)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(updated_date.month, 2)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 18)
        self.assertEqual(updated_date.day, 19)
        self.assertEqual(expires_date.day, 17)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc.")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_state"), "CA")
        self.assertEqual(
            parser_output.get("registrant_address"), "REDACTED FOR PRIVACY"
        )
        self.assertEqual(parser_output.get("registrant_city"), "REDACTED FOR PRIVACY")

    def test_parser_live(self):
        query_output = self.get_txt("live")
        tld = "live"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2015)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(updated_date.month, 10)
        self.assertEqual(expires_date.month, 10)
        self.assertEqual(created_date.day, 19)
        self.assertEqual(updated_date.day, 19)
        self.assertEqual(expires_date.day, 19)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "Nom-iq Ltd. dba COM LAUDE")
        # registrant
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "Amazon Technologies, Inc.",
        )
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_state"), "NV")
        self.assertEqual(
            parser_output.get("registrant_address"), "REDACTED FOR PRIVACY"
        )
        self.assertEqual(parser_output.get("registrant_city"), "REDACTED FOR PRIVACY")

    def test_parser_cat(self):
        query_output = self.get_txt("cat")
        tld = "cat"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2006)
        self.assertEqual(updated_date.year, 2021)
        self.assertEqual(expires_date.year, 2022)
        self.assertEqual(created_date.month, 2)
        self.assertEqual(updated_date.month, 1)
        self.assertEqual(expires_date.month, 2)
        self.assertEqual(created_date.day, 14)
        self.assertEqual(updated_date.day, 15)
        self.assertEqual(expires_date.day, 14)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor")
        # registrant
        self.assertEqual(parser_output.get("registrant_organization"), "Google LLC")
        self.assertEqual(parser_output.get("registrant_country"), "US")
        self.assertEqual(parser_output.get("registrant_state"), "CA")

    def test_parser_ma(self):
        query_output = self.get_txt("ma")
        tld = "ma"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2009)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(updated_date.month, 2)
        self.assertEqual(expires_date.month, 3)
        self.assertEqual(created_date.day, 24)
        self.assertEqual(updated_date.day, 25)
        self.assertEqual(expires_date.day, 24)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "GENIOUS COMMUNICATION")
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 2)

    def test_parser_vg(self):
        query_output = self.get_txt("vg")
        tld = "vg"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 1999)
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 6)
        self.assertEqual(updated_date.month, 12)
        self.assertEqual(expires_date.month, 6)
        self.assertEqual(created_date.day, 5)
        self.assertEqual(updated_date.day, 10)
        self.assertEqual(expires_date.day, 5)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor, Inc.")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 3)

    def test_parser_tk(self):
        query_output = self.get_txt("tk")
        tld = "tk"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        expires_date = parser_output.get("expires")
        self.assertEqual(created_date.year, 2014)
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(expires_date.month, 12)
        self.assertEqual(created_date.day, 17)
        self.assertEqual(expires_date.day, 11)
        # registrar
        self.assertEqual(parser_output.get("registrar"), None)
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "Amazon Technologies, Inc.",
        )
        self.assertEqual(
            parser_output.get("registrant_name"), "Hostmaster Amazon Legal Dept."
        )
        self.assertEqual(len(parser_output.get("name_servers")), 5)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("registrant_country"), "U.S.A.")
        self.assertEqual(parser_output.get("registrant_state"), "Nevada")
        self.assertEqual(parser_output.get("registrant_address"), "P.O. Box 8102")
        self.assertEqual(parser_output.get("registrant_city"), "Reno")

    def test_parser_nl(self):
        query_output = self.get_txt("nl")
        tld = "nl"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        updated_date = parser_output.get("updated")
        self.assertEqual(created_date.year, 1999)
        self.assertEqual(updated_date.year, 2015)
        self.assertEqual(created_date.month, 5)
        self.assertEqual(updated_date.month, 12)
        self.assertEqual(created_date.day, 27)
        self.assertEqual(updated_date.day, 30)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        self.assertEqual(parser_output.get("registrar"), "MarkMonitor Inc.")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_parser_gq(self):
        query_output = self.get_txt("gq")
        tld = "gq"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2014)
        self.assertEqual(created_date.month, 10)
        self.assertEqual(created_date.day, 14)
        # registrar
        self.assertEqual(parser_output.get("registrar"), None)
        self.assertEqual(parser_output.get("registrant_organization"), "Google Inc")
        self.assertEqual(parser_output.get("registrant_name"), "DNS Admin")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("registrant_country"), "U.S.A.")
        self.assertEqual(parser_output.get("registrant_state"), "California")
        self.assertEqual(
            parser_output.get("registrant_address"), "1600 Amphitheatre Parkway"
        )
        self.assertEqual(parser_output.get("registrant_city"), "Mountain View")

    def test_tld_nu(self):
        query_output = self.get_txt("nu")
        tld = "nu"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2011)
        self.assertEqual(created_date.month, 4)
        self.assertEqual(created_date.day, 15)
        updated_date = parser_output.get("updated")
        self.assertEqual(updated_date.year, 2021)
        self.assertEqual(updated_date.month, 2)
        self.assertEqual(updated_date.day, 16)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2022)
        self.assertEqual(expired_date.month, 4)
        self.assertEqual(expired_date.day, 15)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "Domeneshop AS")
        self.assertEqual(parser_output.get("registrant_name"), "DNS1856879")
        self.assertEqual(len(parser_output.get("name_servers")), 3)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("dnssec"), "signed delegation")

    def test_tld_is(self):
        query_output = self.get_txt("is")
        tld = "is"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2000)
        self.assertEqual(created_date.month, 1)
        self.assertEqual(created_date.day, 18)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2022)
        self.assertEqual(expired_date.month, 1)
        self.assertEqual(expired_date.day, 18)
        # registrar
        self.assertEqual(
            parser_output.get("registrant_name"), "Amazon Europe Core S.a.r.l."
        )
        self.assertEqual(
            parser_output.get("registrant_address"),
            "38 avenue John F. Kennedy, LU-L-1855 Luxembourg",
        )
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(parser_output.get("dnssec"), "unsigned delegation")

    def test_tld_cr(self):
        query_output = self.get_txt("cr")
        tld = "cr"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2008)
        self.assertEqual(created_date.month, 3)
        self.assertEqual(created_date.day, 23)
        updated_date = parser_output.get("updated")
        self.assertEqual(updated_date.year, 2021)
        self.assertEqual(updated_date.month, 2)
        self.assertEqual(updated_date.day, 5)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2022)
        self.assertEqual(expired_date.month, 3)
        self.assertEqual(expired_date.day, 24)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "COMLAUDE")
        self.assertEqual(
            parser_output.get("registrant_name"), "Amazon Technologies, Inc."
        )
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "Amazon Technologies, Inc.",
        )
        self.assertEqual(
            parser_output.get("registrant_address"),
            "P.O. Box 8102, Reno, 89507, Nevada, US",
        )
        self.assertEqual(len(parser_output.get("name_servers")), 9)

    def test_tld_cz(self):
        query_output = self.get_txt("cz")
        tld = "cz"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 1997)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(created_date.day, 19)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2021)
        self.assertEqual(expired_date.month, 10)
        self.assertEqual(expired_date.day, 28)
        updated_date = parser_output.get("updated")
        self.assertEqual(updated_date.year, 2017)
        self.assertEqual(updated_date.month, 1)
        self.assertEqual(updated_date.day, 12)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "REG-NOMIQ")
        self.assertEqual(parser_output.get("registrant_name"), "Legal Department")
        self.assertEqual(
            parser_output.get("registrant_organization"),
            "Amazon Europe Holding Technologies SCS",
        )
        self.assertEqual(
            parser_output.get("registrant_address"),
            "65, boulevard Grande-Duchesse Charlotte, Luxembourg City, 1331, LU",
        )
        self.assertEqual(len(parser_output.get("name_servers")), 6)

    def test_tld_gg(self):
        query_output = self.get_txt("gg")
        tld = "gg"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2003)
        self.assertEqual(created_date.month, 4)
        self.assertEqual(created_date.day, 30)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        # registrar
        self.assertEqual(
            parser_output.get("registrar"),
            "MarkMonitor Inc. (http://www.markmonitor.com)",
        )
        # name servers and status
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 4)

    def test_tld_ge(self):
        query_output = self.get_txt("ge")
        tld = "ge"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2006)
        self.assertEqual(created_date.month, 7)
        self.assertEqual(created_date.day, 28)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2021)
        self.assertEqual(expired_date.month, 7)
        self.assertEqual(expired_date.day, 29)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Google LLC")
        # registrar
        self.assertEqual(parser_output.get("registrar"), "proservice ltd")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)

    def test_tld_jp(self):
        query_output = self.get_txt("jp")
        tld = "jp"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2010)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(created_date.day, 22)
        #
        expires_date = parser_output.get("expires")
        self.assertEqual(expires_date.year, 2021)
        self.assertEqual(expires_date.month, 9)
        self.assertEqual(expires_date.day, 30)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "Amazon, Inc.")
        # address
        self.assertEqual(
            parser_output.get("registrant_address"),
            "Meguro-ku, Arco Tower Annex, 8-1, Shimomeguro 1-chome",
        )
        # name servers
        self.assertEqual(len(parser_output.get("name_servers")), 8)

    def test_tld_ax(self):
        query_output = self.get_txt("ax")
        tld = "ax"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2016)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(created_date.day, 8)
        expired_date = parser_output.get("expires")
        self.assertEqual(expired_date.year, 2021)
        self.assertEqual(expired_date.month, 9)
        self.assertEqual(expired_date.day, 8)
        updated_date = parser_output.get("updated")
        self.assertEqual(updated_date.year, 2020)
        self.assertEqual(updated_date.month, 9)
        self.assertEqual(updated_date.day, 5)
        # registrant
        self.assertEqual(parser_output.get("registrant_name"), "xTom GmbH")
        self.assertEqual(parser_output.get("registrant_country"), "Tyskland")
        self.assertEqual(
            parser_output.get("registrant_address"),
            "Kreuzstr.60, 40210, Duesseldorf",
        )
        # registrar
        self.assertEqual(parser_output.get("registrar"), "xTom")
        self.assertEqual(parser_output.get("registrar_url"), "https://xtom.com/")
        # misc
        self.assertEqual(len(parser_output.get("name_servers")), 2)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("domain_name"), "google.ax")

    def test_tld_aw(self):
        query_output = self.get_txt("aw")
        tld = "aw"
        parser_output = self.parser.parse(query_output, tld)
        # confirm dates
        created_date = parser_output.get("created")
        self.assertEqual(created_date.year, 2017)
        self.assertEqual(created_date.month, 9)
        self.assertEqual(created_date.day, 13)
        updated_date = parser_output.get("updated")
        self.assertEqual(updated_date.year, 2018)
        self.assertEqual(updated_date.month, 5)
        self.assertEqual(updated_date.day, 21)
        # registrar
        self.assertEqual(parser_output.get("registrar"), "SETAR N.V.")
        # misc
        self.assertEqual(parser_output.get("dnssec"), "no")
        self.assertEqual(len(parser_output.get("name_servers")), 4)
        self.assertEqual(len(parser_output.get("status")), 1)
        self.assertEqual(parser_output.get("domain_name"), "google.aw")



import asyncwhois


def main():
    non_existent_domain = "dasrewrdgdfgserw34.com"

    try:
        r = asyncwhois.whois_domain(non_existent_domain)
        print(r.query_output)
    except asyncwhois.NotFoundError:
        print("That domain does not exist, so we threw an error.")

    # suppress exception throwing for "not found" domains:
    result = asyncwhois.whois_domain(non_existent_domain, ignore_not_found=True)
    print(result.query_output)

    return


if __name__ == "__main__":
    main()



import time

import asyncwhois


def main():
    urls = [
        'https://www.google.co.uk',
        'en.wikipedia.org/wiki/Pi',
        'https://www.urbandictionary.com/define.php?term=async',
        'twitch.tv',
        'https://www.pcpartpicker.com',
        'https://packaging.python.org/',
        'https://www.amazon.co.jp',
        'github.com/explore',
        'mango.beer'
    ]
    for url in urls:
        whois_data = asyncwhois.whois_domain(url)


if __name__ == '__main__':
    start = time.time()
    main()
    print(f'Done! [{round(time.time() - start, 4)}] seconds.')
